/*
 * Copyright 2012-2015 Intel Corporation All Rights Reserved.
 *
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel
 * Corporation or its suppliers or licensors. Title to the Material
 * remains with Intel Corporation or its suppliers and licensors. The
 * Material may contain trade secrets and proprietary and confidential
 * information of Intel Corporation and its suppliers and licensors,
 * and is protected by worldwide copyright and trade secret laws and
 * treaty provisions. No part of the Material may be used, copied,
 * reproduced, modified, published, uploaded, posted, transmitted,
 * distributed, or disclosed in any way without Intel's prior express
 * written permission.
 *
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise. Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * Unless otherwise agreed by Intel in writing, you may not remove or
 * alter this notice or any other notice embedded in Materials by Intel
 * or Intel's suppliers or licensors in any way.
 */


#ifndef __CLCK3_CLIPS_WRAP__
#define __CLCK3_CLIPS_WRAP__

#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <clips/clips.h>


///stream where clips errors get written to
extern std::stringstream CLIPS_ERR_STREAM;

///converts a multifield object to a std::string, for debugging
std::string multi_to_str(DATA_OBJECT& data);
///gets the std::string name of a clips type
/// throws exception of type is invalid
std::string clips_type_to_str(int type);

///get the clips type based on the std::string name
/// throws exception of type is invalid
int clips_type_to_int(std::string type);

/// convert a boolean to the std::strings "true" or "false"
std::string bool_to_str(bool b);

/// convert the std::strings "true" or "false" to a boolean
bool str_to_bool(std::string s);

///converts a clips instance to a std::string based on the pretty print function
std::string pp_inst(std::string inst_name);

///loads a clips command using the build() function.  the std::string cmd
/// is intended to be a single command in the clips language.
void clips_build_string(void *theEnv, std::string cmd);

///loads a clips command using the eval() function.  the std::string cmd
/// is intended to be a single command in the clips language.
void clips_eval_string(void *theEnv, std::string cmd);

///get class instances from clips
std::vector<void *> get_instances(void *clips_env, std::string class_name);

///helpers for converting multifields to vectors
std::vector<std::string> multifield_to_strvect(DATA_OBJECT& d);
std::vector<void *> multifield_to_voidPvect(DATA_OBJECT &d);
int number_to_int(DATA_OBJECT &d);


//get node instances from knowledge base
struct NodeInstance {
    NodeInstance();
    std::string hostname;
    void *instance_addr;
    std::string instance_name;
};
std::ostream& operator <<(std::ostream& s, const NodeInstance& a);

std::vector<NodeInstance> get_node_instances(void *clips_env);

std::string escape_slot_value(const std::string& v);

void *get_clips_class(void *clips_env, const std::string& class_name);

bool get_slot_type(void *clips_env, void *clips_class, const std::string& slot,
        bool& single, int& data_type);

bool get_slot_data_obj(void *clips_env, void *inst, const std::string& slot,
        DATA_OBJECT& data_obj);

bool get_slot_long(void *clips_env, void *inst, const std::string& slot,
        long& value);

bool get_slot_double(void *clips_env, void *inst, const std::string& slot,
        double& value);


bool get_slot_str(void *clips_env, void *inst, const std::string& slot,
        std::string& value);


bool get_slot_multi_long(void *clips_env, void *inst, const std::string& slot,
        std::vector<long>& values);


bool get_slot_multi_double(void *clips_env, void *inst,
        const std::string& slot,
        std::vector<double>& values);

bool get_slot_multi_str(void *clips_env, void *inst, const std::string& slot,
        std::vector<std::string>& values);


#endif //#ifndef __CLCK3_CLIPS_WRAP__
