/*
 * Copyright 2012-2015 Intel Corporation All Rights Reserved.
 *
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel
 * Corporation or its suppliers or licensors. Title to the Material
 * remains with Intel Corporation or its suppliers and licensors. The
 * Material may contain trade secrets and proprietary and confidential
 * information of Intel Corporation and its suppliers and licensors,
 * and is protected by worldwide copyright and trade secret laws and
 * treaty provisions. No part of the Material may be used, copied,
 * reproduced, modified, published, uploaded, posted, transmitted,
 * distributed, or disclosed in any way without Intel's prior express
 * written permission.
 *
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise. Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * Unless otherwise agreed by Intel in writing, you may not remove or
 * alter this notice or any other notice embedded in Materials by Intel
 * or Intel's suppliers or licensors in any way.
 */


#ifndef __CONNECTOR_CLUSTER_DOT_H
#define __CONNECTOR_CLUSTER_DOT_H


#include <iostream>
#include <map>
#include <string>
#include <vector>

#include <clck.h>


/* 
 * The cluster namespace contains structs which represent the hardware and
 * software configuration of the cluster.  
 *
 * TODO: Represent this data in a class hierarchy.
 */

namespace NsCluster {
    struct Node : clck::Node {
        Node();
        Node(const clck::Node& n);

        std::string architecture;
        unsigned long uptime = 0;
        unsigned long last_reboot_time = 0;
        void *instance;
        std::string instance_name;
    };
    typedef std::vector<Node> Nodes;


    struct Cluster {
        Nodes nodes;
        std::map<std::string,int> hostname_map;
        std::map<std::string,int>::const_iterator
            find(const std::string &hostname) const;
    };
};


std::ostream& operator <<(std::ostream& s, const NsCluster::Node& a);

#endif //#ifndef __CONNECTOR_CLUSTER_DOT_H
