/*
 * Copyright 2012-2015 Intel Corporation All Rights Reserved.
 *
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel
 * Corporation or its suppliers or licensors. Title to the Material
 * remains with Intel Corporation or its suppliers and licensors. The
 * Material may contain trade secrets and proprietary and confidential
 * information of Intel Corporation and its suppliers and licensors,
 * and is protected by worldwide copyright and trade secret laws and
 * treaty provisions. No part of the Material may be used, copied,
 * reproduced, modified, published, uploaded, posted, transmitted,
 * distributed, or disclosed in any way without Intel's prior express
 * written permission.
 *
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise. Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * Unless otherwise agreed by Intel in writing, you may not remove or
 * alter this notice or any other notice embedded in Materials by Intel
 * or Intel's suppliers or licensors in any way.
 */


#ifndef __DATA_PTS_DOT_H__
#define __DATA_PTS_DOT_H__

#include <istream>
#include <string>
#include <vector>
#include <sstream>
#include <memory>

/// data point header
typedef std::vector<std::string> DataPtsHeader;


/// base class
struct DataPt {

    enum TYPES {
        TYPE_UNKNOWN = 0,
        TYPE_LONG    = 1,
        TYPE_DOUBLE  = 2,
        TYPE_STRING  = 3,
        TYPE_MULTI   = 4
    };

    DataPt();
    DataPt(const long& d);
    DataPt(const int& d);
    DataPt(const double& d);
    DataPt(const std::string& d);
    DataPt(const char *d);

    virtual ~DataPt();

    void set_long(const long& v);
    void set_double(const double& v);
    void set_string(const std::string& v);

    int type;
    long val_l;
    double val_d;
    std::string val_s;

    bool is_type_single() const;

    std::vector<std::shared_ptr<DataPt>> vals_dp;
};
std::ostream& operator <<(std::ostream& s, const DataPt& d);

typedef std::vector<DataPt> DataPts;


/// single data point

struct DpLong : public DataPt {

    DpLong();
    DpLong(const long& d);
    DpLong(const int& d);
    DpLong(const DataPt *dp);
};
void operator >>(std::istream& s, DpLong& d);


struct DpDouble : public DataPt { 

    DpDouble();
    DpDouble(const double& d);
    DpDouble(const DataPt *dp);
};
void operator >>(std::istream& s, DpDouble& d);


struct DpString : public DataPt {

    DpString();
    DpString(const std::string& d);
    DpString(const DataPt *dp);
    DpString(const char *d);
};
void operator >>(std::istream& s, DpString& d);


/// multi data point

struct DpMulti : public DataPt {

    DpMulti();
    DpMulti(const DataPt *dp);

    void append(const DataPt *dp);
    void append(const DataPt& dp);
};


#endif //#ifndef __DATA_PTS_DOT_H__
