/*
 * Copyright 2012-2015 Intel Corporation All Rights Reserved.
 *
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel
 * Corporation or its suppliers or licensors. Title to the Material
 * remains with Intel Corporation or its suppliers and licensors. The
 * Material may contain trade secrets and proprietary and confidential
 * information of Intel Corporation and its suppliers and licensors,
 * and is protected by worldwide copyright and trade secret laws and
 * treaty provisions. No part of the Material may be used, copied,
 * reproduced, modified, published, uploaded, posted, transmitted,
 * distributed, or disclosed in any way without Intel's prior express
 * written permission.
 *
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise. Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * Unless otherwise agreed by Intel in writing, you may not remove or
 * alter this notice or any other notice embedded in Materials by Intel
 * or Intel's suppliers or licensors in any way.
 */


#ifndef __CONNECTOR_DB_READ_DOT_H__
#define __CONNECTOR_DB_READ_DOT_H__

#include <map>
#include <string>
#include <vector>

/*
 * Database namespace, contains general structures and manipulation functions
 * for working with the daemon's database.  
 */

namespace NsDb {
    struct Row {
        int row_id;
        time_t row_timestamp;
        std::string provider;
        std::string hostname;
        int num_nodes;
        std::vector<std::string> node_names;
        int exit_status;
        time_t timestamp;
        double duration;
        int encoding;
        int stdout_size;
        std::vector<unsigned char> stdout;
        int stderr_size;
        std::vector<unsigned char> stderr;
        std::string OptionID;
    };
    typedef std::vector<Row> Rows;
    Row init_row();
    std::string row2str(const Row& r);

    enum Encodings { ENC_NONE=0, ENC_BASE64 };

    static std::map<std::string, int> colmap = {
        {"rowid", 0},
        {"row_timestamp", 1},
        {"provider", 2},
        {"hostname", 3},
        {"num_nodes", 4},
        {"node_names", 5},
        {"exit_status", 6},
        {"timestamp", 7},
        {"duration", 8},
        {"encoding", 9},
        {"stdout_size", 10},
        {"STDOUT", 11},
        {"stderr_size", 12},
        {"STDERR", 13},
        {"OptionID", 14}
    };

    /* filter rows for all rows from a host */
    bool get_rows_host(const Rows& in_rows, std::string hostname, 
            Rows& out_rows);

    /* filter frows for the last row from a host */
    bool get_last_row_host(const Rows& in_rows, std::string hostname, 
            Row& out_row);

    const std::string table_name = "clck_1";
};


/*
 * Base class for reading the daemon's database.  Subclasses support 
 * specific database implementations (sqlite, mysql, ...)
 */    

class DbRead {

    public:

        DbRead();
        virtual ~DbRead();

        ///open the database for reading
        ///returns true if successful, otherwise returns false
        virtual bool open(std::string s);

        ///perform the sql query select
        ///returns true if successful, otherwise returns false
        virtual bool select(std::string provider_name, NsDb::Rows& rows,
                std::string where_clause="");

        virtual bool select(const std::string query, NsDb::Rows& rows,
                const std::map<std::string, int>& columns);

        ///perform select query to get the last row for a single data
        ///provider on a single host
        ///writes output lines for the data point to the vector out_lines
        ///returns true if successful, otherwise returns false
        virtual bool get_last_row_provider(std::string provider, 
                std::string hostname, 
                std::vector<std::string>& out_lines, NsDb::Row& row);

        ///perform select query to the latest rows for each host for a
        ///single data provider
        ///returns true if successful, otherwise returns false
        virtual bool get_latest_rows_provider(std::string provider,
                NsDb::Rows& rows);

    protected:
        std::string name;

};


#endif //#ifndef __DB_READ_DOT_H__
