/*
 * Copyright 2012-2014 Intel Corporation All Rights Reserved.
 *
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel
 * Corporation or its suppliers or licensors. Title to the Material
 * remains with Intel Corporation or its suppliers and licensors. The
 * Material may contain trade secrets and proprietary and confidential
 * information of Intel Corporation and its suppliers and licensors,
 * and is protected by worldwide copyright and trade secret laws and
 * treaty provisions. No part of the Material may be used, copied,
 * reproduced, modified, published, uploaded, posted, transmitted,
 * distributed, or disclosed in any way without Intel's prior express
 * written permission.
 *
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise. Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * Unless otherwise agreed by Intel in writing, you may not remove or
 * alter this notice or any other notice embedded in Materials by Intel
 * or Intel's suppliers or licensors in any way.
 */


#ifndef __NETWORK_DEVICE_DOT_H__
#define __NETWORK_DEVICE_DOT_H__

#include "extension.h"


class NetworkDevice : public Extension {

 public:
    const static std::string intel_vendor_id;
    const static std::string mellanox_vendor_id;
    const static std::string qlogic_vendor_id;

    struct Device {
        Device();
        std::string name;
        std::string devpath;

        std::vector<std::string> ip4_addrs;
        std::vector<std::string> ip6_addrs;
        std::string mac;
        std::string mac_fmt;

        std::vector<std::string> dapl_providers;
        std::vector<std::string> ofa_adapters;
        std::vector<std::string> tmi_providers;
        std::string link_layer;
        std::string port_guid;
        std::string driver;
        std::string driver_version;
        int rx_usecs;
    };
    typedef std::vector<Device> Devices;


    struct Adapter {
        std::string str_verbose();
        std::string pci_addr;
        std::string vendor_id;
        std::string model;
        std::string model_id;
        Devices devices;
        std::string instance_name;
        std::string driver;
        std::vector<std::string> kernel_mods;
    };
    typedef std::vector<Adapter> Adapters;

    NetworkDevice(DbRead& db_read, void *clips_env);
    virtual bool parse();

    static void get_net_instance_name(void *clips_env, Adapter& adapter,
                                      std::string hostname);

    static bool net_udevadm_parse(DbRead& db_read, const std::string& hostname,
                                  Devices& devices);

    static int get_net_device_idx(const Devices& devices,
                                  const std::string& name);

    static std::string get_link_str(const std::string& s);

    static bool net_ipaddr_parse(DbRead& db_read, const std::string& hostname,
                                 Devices& devices);

    static int get_net_adapter_idx(const Adapters& adapters,
                                   const std::string& pci_addr);

    static int net_lspci_parse_adapter(const std::vector<std::string>& lines,
                                       int line_idx,
                                       Adapter& adapter);

    static bool net_lspci_parse(DbRead& db_read, const std::string& hostname,
                                Adapters& adapters, Devices& devices);

    static bool mac_eq_guid(const std::string& mac, std::string guid);

    static bool net_ibstat_parse(DbRead& db_read, const std::string& hostname,
                                 Devices& devices);

    static bool net_datconf_parse(DbRead& db_read,
                                  const std::string& hostname,
                                  Adapters& adapters);

    static bool net_tmiconf_parse(DbRead& db_read,
                                  const std::string& hostname,
                                  Adapters& adapters);

    static bool net_ethtool_parse(DbRead& db_read,
                                  const std::string& hostname,
                                  Adapter& adapter);

    static bool net_ethtool_coalesce_parse(DbRead& db_read,
                                           const std::string& hostname,
                                           Adapter& adapter);

    static bool net_ofedinfo_parse(DbRead& db_read,
                                   const std::string& hostname,
                                   Adapter& adapter);

    static bool net_driver(DbRead& db_read, const std::string& hostname,
                           Adapters& adapters);

    bool get_dapl_expected_perf(const std::string& hostname,
                                const std::string& dapl_provider,
                                double& bandwidth, double& latency);

    bool get_ofa_expected_perf(const std::string& hostname,
                               const std::string& ofa_adapter,
                               double& bandwidth, double& latency);

    bool get_tmi_expected_perf(const std::string& hostname,
                               const std::string& tmi_provider,
                               double& bandwidth, double& latency);

    const std::map<std::string, Adapters>& get_node_map();

 protected:
    std::map<std::string, Adapters> node_map;

 private:
    typedef Extension super;

};


std::ostream& operator <<(std::ostream& s, const NetworkDevice::Device& a);
std::ostream& operator <<(std::ostream& s, const NetworkDevice::Adapter& a);

#endif //#ifndef __NETWORK_DEVICE_DOT_H__