/*
 * Copyright 2012-2015 Intel Corporation All Rights Reserved.
 *
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel
 * Corporation or its suppliers or licensors. Title to the Material
 * remains with Intel Corporation or its suppliers and licensors. The
 * Material may contain trade secrets and proprietary and confidential
 * information of Intel Corporation and its suppliers and licensors,
 * and is protected by worldwide copyright and trade secret laws and
 * treaty provisions. No part of the Material may be used, copied,
 * reproduced, modified, published, uploaded, posted, transmitted,
 * distributed, or disclosed in any way without Intel's prior express
 * written permission.
 *
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise. Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * Unless otherwise agreed by Intel in writing, you may not remove or
 * alter this notice or any other notice embedded in Materials by Intel
 * or Intel's suppliers or licensors in any way.
 */


#ifndef __CONNECTOR_TRANSFORM_DOT_H__
#define __CONNECTOR_TRANSFORM_DOT_H__


/**
 * Transformers are generic data processors
 * Their functionality is as follows:
 *   Parse data from the data providers
 *   Filter the data
 *   Populate the corresponding clips objects/instances
 */


#include "db_read.h"
#include "data_pt.h"

#include <clips/clips.h>


class Transform {

    public:

        enum ROUTES {
            ROUTE_NONE      = 0,
            ROUTE_CSV       = 1,
            ROUTE_CLIPS_STR = 2
        };

        Transform(DbRead& db_read, void *clips_env);
        virtual ~Transform();

        std::string get_name();

        ///read data from the input source
        virtual bool parse();

        //set output file stream
        void set_csv_output_stream(std::ostream *output_stream);

        //clips instance name
        void set_name(const std::string& name);

        //clips instance name
        void set_clips_class_name(const std::string& name);

        //column header for data points
        void set_header(const DataPtsHeader& h);

        int get_route_mask();
        void set_route_mask(int mask);
        bool is_route_active(int route);

        //output a row of data points
        virtual void route(const DataPts& d);
        void route_csv(const DataPts& d);
        void route_clips_str(const DataPts& d);

    protected:

        std::string name;
        DbRead& db_read;
        void *clips_env;

        std::string clips_class_name;
        std::vector<std::string> data_header;

        std::ostream *csv_ostream;

        int route_mask;
};


#endif ///#ifndef __CONNECTOR_TRANSFORM_DOT_H__
