/*
 * Copyright 2012-2015 Intel Corporation All Rights Reserved.
 *
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel
 * Corporation or its suppliers or licensors. Title to the Material
 * remains with Intel Corporation or its suppliers and licensors. The
 * Material may contain trade secrets and proprietary and confidential
 * information of Intel Corporation and its suppliers and licensors,
 * and is protected by worldwide copyright and trade secret laws and
 * treaty provisions. No part of the Material may be used, copied,
 * reproduced, modified, published, uploaded, posted, transmitted,
 * distributed, or disclosed in any way without Intel's prior express
 * written permission.
 *
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise. Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * Unless otherwise agreed by Intel in writing, you may not remove or
 * alter this notice or any other notice embedded in Materials by Intel
 * or Intel's suppliers or licensors in any way.
 */

#ifndef __CLCK3_UTIL_NODE_DOT_H__
#define __CLCK3_UTIL_NODE_DOT_H__


#include <string>
#include <map>
#include <vector>
#include <iostream>

#include <clck.h>

namespace clck {

    namespace node {

        extern std::map<std::string, Node::role_t> role_str_map;
        extern std::map<Node::role_t, std::string> role_int_map;

        std::vector<Node::role_t>
            role_convert_strs(const std::vector<std::string>& roles);

        std::vector<std::string>
            role_convert_ints(const std::vector<Node::role_t>& roles);


        struct Nodelist {
            static bool hostname_valid(const std::string& hostname);

            Nodelist();
            Nodelist(std::istream& in);

            void parse(std::istream& in);

            //process attribute of the form:  key:value
            static bool parse_attr_nospace(std::string& key, std::string& val);

            //process attribute of the form:  key: value
            static bool parse_attr_space(std::stringstream *ss, std::string& key,
                                         std::string& val, int linenum=0);

            static void process_attr_pair(const std::string& key,
                                          const std::string& val, Node& node,
                                          int linenum=0);

            static bool parse_attrs(const std::string& line, Node& node,
                                    int linenum=0);

            void add_node(const Node& node, int linenum=0);
            bool node_in_list(const std::string& hostname);
            const Node& get_node(const std::string& hostname);

            std::vector<Node> nodes;
            std::map<std::string, int> name_map;	
        };

    }; //namespace node

};  //namespace clck

std::ostream& operator <<(std::ostream& s, const clck::Node& n);
std::ostream& operator <<(std::ostream& s, const clck::node::Nodelist& n);


#endif //#ifndef __CLCK3_UTIL_NODE_DOT_H__
