; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defclass CPU
  "This class represents the CPU on a system."
  (is-a BASE_NODE DATABASE MULTISET)
  (role concrete)
  (pattern-match reactive)

  (slot family (type INTEGER) (default 0))
  (multislot flags (type SYMBOL))
  (slot model (type INTEGER) (default 0))
  (slot model-name (type STRING) (default ""))
  (slot stepping (type INTEGER) (default 0))
  (slot vendor-id (type STRING) (default "")))

(defmessage-handler CPU init after ()
  (send ?self add (send ?self uniformity-key model-name) ?self:model-name))

(defmessage-handler CPU uniformity-key (?v)
  "Return a key that is unique for each combination of subcluster,
   role, and architecture."
  (bind ?architecture nil)
  (bind ?role nil)
  (bind ?subcluster nil)

  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then
    (bind ?i (nth$ 1 ?ins))
    (bind ?architecture (send ?i get-architecture))
    (bind ?subcluster (send ?i get-subcluster))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role enhanced))))

  (bind ?key (sym-cat (class ?self) + ?subcluster + ?role + ?architecture
                      + ?v))
  (return ?key))
