; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defclass DGEMM
  "This class represents the data collected from a run of the dgemm
   benchmark.  Since the 'D' in DGEMM is double precision, all the
   units are implicitly double precision.  Instances of the class are
   created, and the data slots are populated, by the connector
   component."
  (is-a BASE_NODE BASE_TIMESTAMP STATISTICS DATABASE)
  (role concrete)
  (pattern-match reactive)

  (slot flops (type NUMBER) (default 0) ; units = double precision floating
        (range 0 ?VARIABLE))            ;         point ops per second
  (slot peak (type NUMBER) (default 0)  ; units = same as flops slot
        (range 0 ?VARIABLE)))

(defmessage-handler DGEMM init after ()
  (if (> ?self:flops 0) then
    (send ?self add (send ?self statistics-key) ?self:flops)))

(defmessage-handler DGEMM statistics-key ()
  "Return a key that is unique for each combination of subcluster,
   role, and architecture, e.g., DGEMM+sub+compute+x86_64."
  (bind ?role nil)
  (bind ?subcluster nil)

  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then
    (bind ?i (nth$ 1 ?ins))
    (bind ?subcluster (send ?i get-subcluster))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role enhanced))))

  (bind ?key (sym-cat (class ?self) + ?subcluster + ?role + ?self:architecture))
  (return ?key))
