; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defclass ENVIRONMENT
  (is-a BASE_NODE MULTISET DATABASE)
  (role concrete)
  (pattern-match reactive)

  ; This slot represents the environment variable
  (slot variable (type SYMBOL) (default UNKNOWN))

  ; This slot represents the value of the environment variable.  In
  ; the case of PATH like variables, this is the trimmed value based
  ; on what was found on pure compute nodes For regular environment
  ; variables this is equal to the true-value
  (slot value (type STRING) (default ""))

  ; The unmodified value of the environment variable as seen in the
  ; database
  (slot true-value (type STRING) (default ""))

  ; This is set by the connector. If the connector sets it to TRUE,
  ; this particular variable will be checked for uniformity.  The
  ; connector will decide based on node roles.
  (slot require-uniformity (type SYMBOL) (default UNKNOWN)))

(defmessage-handler ENVIRONMENT uniformity-key ()
  (bind ?role nil)
  (bind ?subcluster nil)
  (bind ?arch nil)
  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then 
    (bind ?i (nth$ 1 ?ins))
    (bind ?arch (send ?i get-architecture))
    (bind ?subcluster (send ?i get-subcluster)))

  (bind ?key (sym-cat (class ?self) + ?subcluster + ?arch + ?self:variable))
  (return ?key))

(defmessage-handler ENVIRONMENT init after ()
  (if (eq ?self:require-uniformity FALSE) then
    (return 0))

  (send ?self add (send ?self uniformity-key) ?self:value))
