; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defclass HPL
  (is-a CLUSTER IMPI STATISTICS DATABASE)
  (role concrete)
  (pattern-match reactive)

  (slot gflops (type NUMBER) (default 0))
  (slot n (type INTEGER) (default 0))
  (slot nb (type INTEGER) (default 0))
  (slot p (type INTEGER) (default 0))
  (slot q (type INTEGER) (default 0))
  (slot residual (type SYMBOL) (allowed-values PASSED FAILED UNKNOWN)
        (default UNKNOWN))
  (slot rmax (type NUMBER) (default 0))
  (slot execution-time (type NUMBER) (default 0))
  (slot provider (type SYMBOL) (allowed-values CLUSTER PAIRWISE)
        (default PAIRWISE))
  (slot number-of-coprocessors (type INTEGER) (default 0))
  (slot OptionID (type SYMBOL)))

(defmessage-handler HPL statistics-key ()
  "Return a key that is unique for each combination of subcluster,
   role, option id, and fabric."
  (bind ?role1 nil)
  (bind ?role2 nil)
  (bind ?subcluster1 nil)
  (bind ?subcluster2 nil)

  (bind ?ins1 (find-instance ((?n NODE))
                             (eq ?n:node_id (nth$ 1 ?self:node_id))))
  (if (= (length ?ins1) 1) then
    (bind ?i (nth$ 1 ?ins1))
    (bind ?subcluster1 (send ?i get-subcluster))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role1 compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role1 enhanced))))

  (bind ?ins2 (find-instance ((?n NODE))
                             (eq ?n:node_id (nth$ 2 ?self:node_id))))
  (if (= (length ?ins2) 1) then
    (bind ?i (nth$ 1 ?ins2))
    (bind ?subcluster2 (send ?i get-subcluster))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role2 compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role2 enhanced))))

  (switch ?self:i_mpi_fabrics
          (case dapl then
                (bind ?subfabric ?self:i_mpi_dapl_provider))
          (case ofa then
                (bind ?subfabric ?self:i_mpi_ofa_adapter_name))
          (case shm then
                (bind ?subfabric nil))
          (case shm:dapl then
                (bind ?subfabric ?self:i_mpi_dapl_provider))
          (case shm:ofa then
                (bind ?subfabric ?self:i_mpi_ofa_adapter_name))
          (case shm:tcp then
                (bind ?subfabric ?self:i_mpi_tcp_netmask))
          (case shm:tmi then
                (bind ?subfabric ?self:i_mpi_tmi_provider))
          (case tcp then
                (bind ?subfabric ?self:i_mpi_tcp_netmask))
          (case tmi then
                (bind ?subfabric ?self:i_mpi_tmi_provider))
          (default bind ?subfabric nil))

  (bind ?key (sym-cat (class ?self)
                      - ?subcluster1 - ?subcluster2 - ?role1 - ?role2
                      - ?self:i_mpi_fabrics - ?subfabric
                      - ?self:number-of-coprocessors - ?self:OptionID))
  (return ?key))

(defmessage-handler HPL init after()
  (if (eq ?self:provider PAIRWISE) then
    (if (eq ?self:residual PASSED) then
      (send ?self add (send ?self statistics-key) ?self:gflops))))

