; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.


(defclass IMB_PINGPONG
  (is-a NODE_PAIR IMPI STATISTICS DATABASE)
  (role concrete)
  (pattern-match reactive)

  ; architecture of endpoints
  (multislot arch (type SYMBOL) (allowed-values k1om x86_64)
             (cardinality 2 2) (default (create$ x86_64 x86_64)))

  ; bandwidth in megabytes/secs
  (slot bandwidth (type FLOAT))

  ; local if node pair are on the same host, else remote
  (slot endpoints (type SYMBOL) (allowed-values local remote) (default remote))

  ; expected values for absolute threshold tests
  (slot expected-bandwidth (type FLOAT) (default 0.0))
  (slot expected-latency (type FLOAT) (default 1e6))

  ; latency in microseconds
  (slot latency (type FLOAT))
  (slot option_id (type STRING) (default "")))

(defmessage-handler IMB_PINGPONG statistics-key (?suffix)
  "Return a key that is unique for each combination of subcluster,
   role, architecture, option id, endpoint, and fabric."
  (bind ?role1 nil)
  (bind ?role2 nil)
  (bind ?subcluster1 nil)
  (bind ?subcluster2 nil)

  (bind ?ins1 (find-instance ((?n NODE))
                             (eq ?n:node_id (nth$ 1 ?self:node_id))))
  (if (= (length ?ins1) 1) then
    (bind ?i (nth$ 1 ?ins1))
    (bind ?subcluster1 (send ?i get-subcluster))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role1 compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role1 enhanced))))

  (bind ?ins2 (find-instance ((?n NODE))
                             (eq ?n:node_id (nth$ 2 ?self:node_id))))
  (if (= (length ?ins2) 1) then
    (bind ?i (nth$ 1 ?ins2))
    (bind ?subcluster2 (send ?i get-subcluster))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role2 compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role2 enhanced))))

  (switch ?self:i_mpi_fabrics
          (case dapl then
                (bind ?subfabric ?self:i_mpi_dapl_provider))
          (case ofa then
                (bind ?subfabric ?self:i_mpi_ofa_adapter_name))
          (case shm:dapl then
                (bind ?subfabric ?self:i_mpi_dapl_provider))
          (case shm:ofa then
                (bind ?subfabric ?self:i_mpi_ofa_adapter_name))
          (case shm:tcp then
                (bind ?subfabric ?self:i_mpi_tcp_netmask))
          (case shm:tmi then
                (bind ?subfabric ?self:i_mpi_tmi_provider))
          (case tcp then
                (bind ?subfabric ?self:i_mpi_tcp_netmask))
          (case tmi then
                (bind ?subfabric ?self:i_mpi_tmi_provider))
          (default (bind ?subfabric nil)))

  (bind ?key (sym-cat (class ?self) _ ?subcluster1 _ ?subcluster2 _
                      ?role1 _ ?role2 _ ?self:option_id _ (nth$ 1 ?self:arch) _
                      (nth$ 2 ?self:arch) _ ?self:endpoints _
                      ?self:i_mpi_fabrics _ ?subfabric _ ?suffix))
  (return ?key))

(defmessage-handler IMB_PINGPONG init after ()
  (if (eq ?self:endpoints remote) then
    (send ?self add (send ?self statistics-key bandwidth) ?self:bandwidth)
    (send ?self add (send ?self statistics-key latency) ?self:latency)))
