; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defclass INFINIBAND_CONFIG
  "Class for InfiniBand software configuration settings."
  (is-a BASE_NODE DATABASE MULTISET)
  (role concrete)
  (pattern-match reactive)

  (slot memlock (type INTEGER) (default 0)) ; The value -1 means "unlimited"
  (slot ofed-version (type STRING) (default "unknown")))

(defmessage-handler INFINIBAND_CONFIG init after ()
  (send ?self add (send ?self multiset-key memlock) ?self:memlock)
  (send ?self add (send ?self multiset-key ofed-version) ?self:ofed-version))

(defmessage-handler INFINIBAND_CONFIG multiset-key (?v)
  ; defaults
  (bind ?subcluster default)
  (bind ?role compute)
  (bind ?architecture x86_64)

  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then
    (bind ?i (nth$ 1 ?ins))
    (bind ?subcluster (send ?i get-subcluster))
    (bind ?architecture (send ?i get-architecture))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role enhanced))))

  (bind ?key (sym-cat (class ?self) + ?subcluster + ?role + ?architecture + ?v))
  (return ?key))

(defclass INFINIBAND_HCA
  "Class for InfiniBand HCA configuration settings, i.e., ibstat."
  (is-a BASE_NODE BASE_TIMESTAMP DATABASE MULTISET)
  (role concrete)
  (pattern-match reactive)

  (slot base-lid (type INTEGER) (default 0))
  (slot ca (type SYMBOL) (default unknown))
  (slot ca-type (type SYMBOL) (default unknown))
  (slot capability-mask (type SYMBOL) (default unknown))
  (slot firmware-version (type SYMBOL) (default unknown))
  (slot hardware-version (type SYMBOL) (default unknown))
  (slot link-layer (type SYMBOL) (default unknown))
  (slot lmc (type INTEGER) (default 0))
  (slot node-guid (type SYMBOL) (default unknown))
  (slot num-ports (type INTEGER) (default 0))
  (slot physical-state (type SYMBOL) (default unknown))
  (slot port (type INTEGER) (default 0))
  (slot port-guid (type SYMBOL) (default unknown))
  (slot rate (type NUMBER) (default 0))
  (slot sm-lid (type INTEGER) (default 0))
  (slot state (type SYMBOL) (default unknown))
  (slot system-image-guid (type SYMBOL) (default unknown)))

(defmessage-handler INFINIBAND_HCA init after ()
  (send ?self add (send ?self multiset-key ca-type) ?self:ca-type)
  (send ?self add (send ?self multiset-key firmware-version)
        ?self:firmware-version)
  (send ?self add (send ?self multiset-key hardware-version)
        ?self:hardware-version)
  (send ?self add (send ?self multiset-key physical-state) ?self:physical-state)
  (send ?self add (send ?self multiset-key rate) ?self:rate)
  (send ?self add (send ?self multiset-key state) ?self:state))

(defmessage-handler INFINIBAND_HCA multiset-key (?v)
  ; defaults
  (bind ?subcluster default)
  (bind ?role compute)
  (bind ?architecture x86_64)

  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then
    (bind ?i (nth$ 1 ?ins))
    (bind ?subcluster (send ?i get-subcluster))
    (bind ?architecture (send ?i get-architecture))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role enhanced))))

  (bind ?key (sym-cat (class ?self) + ?self:ca + ?self:port + ?subcluster +
                      ?role + ?architecture + ?v))
  (return ?key))

(defclass INFINIBAND_PCI_DEVICE
  "Class for InfiniBand HCA PCI device data, i.e., lspci."
  (is-a BASE_NODE BASE_TIMESTAMP DATABASE MULTISET)
  (role concrete)
  (pattern-match reactive)

  (slot device (type STRING) (default "unknown"))
  (slot device_id (type SYMBOL) (default unknown))
  (slot driver (type SYMBOL) (default unknown))
  (slot module (type SYMBOL) (default unknown))
  (slot physlot (type SYMBOL) (default unknown))
  (slot rev (type SYMBOL) (default unknown))
  (slot slot (type SYMBOL) (default unknown))
  (slot vendor (type STRING) (default "unknown"))
  (slot vendor_id (type SYMBOL) (default unknown)))

(defmessage-handler INFINIBAND_PCI_DEVICE init after ()
  (send ?self add (send ?self multiset-key device) ?self:device)
  (send ?self add (send ?self multiset-key driver) ?self:driver)
  (send ?self add (send ?self multiset-key module) ?self:module)
  (send ?self add (send ?self multiset-key physlot) ?self:physlot)
  (send ?self add (send ?self multiset-key rev) ?self:rev)
  (send ?self add (send ?self multiset-key vendor) ?self:vendor))

(defmessage-handler INFINIBAND_PCI_DEVICE multiset-key (?v)
  ; defaults
  (bind ?subcluster default)
  (bind ?role compute)
  (bind ?architecture x86_64)

  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then
    (bind ?i (nth$ 1 ?ins))
    (bind ?subcluster (send ?i get-subcluster))
    (bind ?architecture (send ?i get-architecture))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role enhanced))))

  (bind ?key (sym-cat (class ?self) + ?self:slot + ?subcluster +
                      ?role + ?architecture + ?v))
  (return ?key))
