; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defclass IOZONE
  (is-a BASE_NODE STATISTICS TIMESTAMP DATABASE)
  (role concrete)
  (pattern-match reactive)

  ; This value is read and updated by the connector by parsing the
  ; STDOUT. This value is always positive, and 0.0 if the connector's
  ; regex parsing didn't return anything
  (slot bandwidth (type NUMBER) (default 0))

  ; This value is set to true if the bandwidth is greater than 0.0 in
  ; the initialization message handler. This is then used to calculate
  ; statistics.
  (slot valid (type SYMBOL) (allowed-symbols TRUE FALSE UNKNOWN)
        (default UNKNOWN))

  ; This is the standard out as seen in the database.
  (slot stdout (type STRING)))

(defmessage-handler IOZONE init after ()
  "Add STATISTICS key based on the node architecture, e.g.,
   IOZONE-x86_64"
  (if (> ?self:bandwidth 0.0)
      then
    (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
    (send ?self add (send ?self statistics-key) ?self:bandwidth)
    (send ?self put-valid TRUE)))

(defmessage-handler IOZONE statistics-key ()
  (bind ?role nil)
  (bind ?subcluster nil)
  (bind ?arch nil)
  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then 
    (bind ?i (nth$ 1 ?ins))
    (bind ?arch (send ?i get-architecture))
    (bind ?subcluster (send ?i get-subcluster))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role enhanced))))

  (bind ?key (sym-cat (class ?self) + ?subcluster + ?role + ?arch))
  (return ?key))

    

