; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defclass MICINFO_HOST
  "This class represents micinfo test results for systems with
   Intel(R) Xeon Phi(TM) coprocessors."
  (is-a BASE_NODE MULTISET TIMESTAMP DATABASE)
  (role concrete)
  (pattern-match reactive)

  (slot driver_version (type STRING) (default ?NONE))
  (slot mpss_version (type STRING) (default ?NONE))
  (slot stderr (type STRING)))

(defmessage-handler MICINFO_HOST init after ()
  "Add MULTISET keys for MICINFO_HOST.  Skip for null version values"
  (if (> (str-length ?self:driver_version) 0) then
    (send ?self add (send ?self uniformity-key driver_version)
          ?self:driver_version))
  (if (> (str-length ?self:mpss_version) 0) then
    (send ?self add (send ?self uniformity-key mpss_version)
          ?self:mpss_version)))

(defmessage-handler MICINFO_HOST uniformity-key (?v)
  (bind ?role nil)
  (bind ?subcluster nil)
  (bind ?arch nil)
  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then
    (bind ?i (nth$ 1 ?ins))
    (bind ?arch (send ?i get-architecture))
    (bind ?subcluster (send ?i get-subcluster))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role enhanced))))

  (bind ?key (sym-cat (class ?self) - ?subcluster - ?role - ?arch - ?v))
  (return ?key))

(defclass MICINFO_DEVICE
  "This class represents the micinfo test results for Intel(R) Xeon
   Phi(TM) coprocessors installed in the host which the slot host
   points to."
  (is-a BASE_NODE MULTISET TIMESTAMP DATABASE)
  (role concrete)
  (pattern-match reactive)

  (slot device_id (type STRING))
  (slot host (type INSTANCE) (allowed-classes MICINFO_HOST))
  (slot Flash_Version (type STRING))
  (slot SMC_Firmware_Version (type STRING))
  (slot SMC_Boot_Loader_Version (type STRING))
  (slot uOS_Version (type STRING))
  (slot Coprocessor_Stepping (type STRING))
  (slot Die_Temp (type NUMBER)))

(defmessage-handler MICINFO_DEVICE init after ()
  "Add MULTISET keys for MICINFO_DEVICE.  Skip for null values"
  (if (> (str-length ?self:Flash_Version) 0) then
    (send ?self add (send ?self uniformity-key Flash_Version)
          ?self:Flash_Version))
  (if (> (str-length ?self:SMC_Firmware_Version) 0) then
    (send ?self add (send ?self uniformity-key SMC_Firmware_Version)
          ?self:SMC_Firmware_Version))
  (if (> (str-length ?self:SMC_Boot_Loader_Version) 0) then
    (send ?self add (send ?self uniformity-key SMC_Boot_Loader_Version)
          ?self:SMC_Boot_Loader_Version))
  (if (> (str-length ?self:uOS_Version) 0) then
    (send ?self add (send ?self uniformity-key uOS_Version) ?self:uOS_Version))
  (if (> (str-length ?self:Coprocessor_Stepping) 0) then
    (send ?self add (send ?self uniformity-key Coprocessor_Stepping)
          ?self:Coprocessor_Stepping)))

(defmessage-handler MICINFO_DEVICE uniformity-key (?v)
  (bind ?role nil)
  (bind ?subcluster nil)
  (bind ?arch nil)
  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then
    (bind ?i (nth$ 1 ?ins))
    (bind ?arch (send ?i get-architecture))
    (bind ?subcluster (send ?i get-subcluster))
    (if (member$ comput (send ?i get-role)) then
      (bind ?role compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role enhanced))))

  (bind ?key (sym-cat (class ?self) - ?subcluster - ?role - ?arch - ?v))
  (return ?key))
