; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.


(defclass MPI_LOCAL
  "This class contains 2 sets of data - 1. The location of `mpirun` as
   output by `which mpirun` 2. The number of times the phrase `Hello
   world: rank [0-9] of [0-9]` occurs."
  (is-a BASE_NODE IMPI MULTISET TIMESTAMP DATABASE)
  (role concrete)
  (pattern-match reactive)

  ; The location of `mpirun` as output by `which mpirun` Note: If this
  ; fails, the assumption is that the Intel(R) Cluster Runtimes rules
  ; would have caught it already.
  (slot mpi_path (type STRING) (default ?NONE))

  ; This is populated by the connector based on the number of times
  ; the phrase `Hello World: rank [0-9] of [0-9]` occurs. Note: 4 is
  ; hard coded, so we expect 4 lines to be here
  (slot valid_lines (type INTEGER) (default 0)))

(defmessage-handler MPI_LOCAL init after()
  "Add MULTISET keys for MPI_LOCAL. Skip cases where there was no mpi_path"
  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (> (str-length ?self:mpi_path) 0) then
    (send ?self add (send ?self uniformity-key) ?self:mpi_path)))

(defmessage-handler MPI_LOCAL uniformity-key ()
  (bind ?role nil)
  (bind ?subcluster nil)
  (bind ?arch nil)
  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then
    (bind ?i (nth$ 1 ?ins))
    (bind ?arch (send ?i get-architecture))
    (bind ?subcluster (send ?i get-subcluster))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role enhanced))))

  (bind ?key (sym-cat (class ?self) + ?subcluster + ?role + ?arch))
  (return ?key))
