; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defclass RPM
  "RPM class used to store the rpm's that are installed on the nodes.
   For every node and every rpm there should be one defclass that
   indicates if the rpm is installed or not."
  (is-a BASE_NODE MULTISET DATABASE)
  (role concrete)
  (pattern-match reactive)
  (slot rpm (type STRING))
  (slot value (type SYMBOL) (allowed-values TRUE FALSE)))




(defmessage-handler RPM init after ()
  "Create uniformity keys for missing and extra rpms"
    ;missing rpms list
    (send ?self uniformity-key missing)
    
    ;extra rpms list, exclude head node
    (bind ?instances (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
    (if (= (length ?instances) 1) then
      (bind ?node (nth$ 1 ?instances))
      (if (not (member$ head (send ?node get-role))) then
        (send ?self uniformity-key extra))))
      
(defmessage-handler RPM uniformity-key (?suffix)
  (bind ?instances (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?instances) 1) then
    (bind ?node (nth$ 1 ?instances))
    
    (bind ?subcluster nil)
    (bind ?arch nil)
    
    (bind ?arch (send ?node get-architecture))
    (bind ?subcluster (send ?node get-subcluster))
    
    (send ?self add (sym-cat (class ?self) - ?self:rpm - ?arch - ?subcluster
                             - ?suffix) ?self:value)))
