; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defclass STREAM
  "This class represents the data collected from a run of the stream
   benchmark. Note, only the maximum value from a Triad run is saved."
  (is-a BASE_NODE STATISTICS TIMESTAMP DATABASE)
  (role concrete)
  (pattern-match reactive)

  ; This slot gets filled by the connector based on the maximum Triad
  ; bandwidth value
  (slot triad (type NUMBER) (default 0))
  ; In this slot, we will look for the sequence "libiomp5.so not
  ; found".  If that is the case, we will tell the user Intel Runtimes
  ; need to be installed
  (slot stderr (type STRING) )
  ; In this slot, we will look for "Solution validates" and "Failed
  ; Validation" . If "Failed Validation" is found, we will tell the
  ; user to collect another set of data since something went wrong. If
  ; solution validates, we can use the current reading.
  (slot stdout (type STRING) )
  ; CLIPS populated this slot depending on whether "FAILED" or "VALID"
  ; is found in the stdout
  (slot valid (type SYMBOL) (allowed-symbols TRUE FALSE UNKNOWN)
        (default UNKNOWN)))

(defmessage-handler STREAM init after ()
  "Add STATISTICS key based on the node architecture, e.g.,
  STREAM-x86_64"
  (if (str-index "Solution Validates" ?self:stdout)
      then
    (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
    (send ?self add (send ?self statistics-key) ?self:triad)
    (send ?self put-valid TRUE)))

(defmessage-handler STREAM statistics-key ()
  (bind ?role nil)
  (bind ?subcluster nil)
  (bind ?arch nil)
  (bind ?ins (find-instance ((?n NODE)) (eq ?n:node_id ?self:node_id)))
  (if (= (length ?ins) 1) then
    (bind ?i (nth$ 1 ?ins))
    (bind ?arch (send ?i get-architecture))
    (bind ?subcluster (send ?i get-subcluster))
    (if (member$ compute (send ?i get-role)) then
      (bind ?role compute)
      else (if (member$ enhanced (send ?i get-role)) then
             (bind ?role enhanced))))

  (bind ?key (sym-cat (class ?self) + ?subcluster + ?role + ?arch))
  (return ?key))
