; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defclass CONFIG
  "Global configuration class.
   The CONFIG class allows users to parameterize the analysis engine
   in a single place.  There should only be one instance of this class
   created per clips environment."
  (is-a USER)
  (role concrete)
  (pattern-match reactive)

  ; The list of checks to be performed.  One to one mapping with
  ; connector extensions of the same name.
  (multislot clck-checks (type SYMBOL)
             (default (create$ all_to_all cpu dgemm environment
                               ethernet heartbeat hpl icr_cluster
                               icr_version imb_pingpong infiniband
                               intel_cluster_runtimes iozone java
                               kernel kernel_param libraries lshw
                               memory miccheck micinfo mount
                               mpi_internode mpi_local ntp offload_phi
                               perl process python rpm shells storage
                               stream tcl x11_tools)))

  ; The checking mode.  Compliance checks compliance to the Intel(R)
  ; Cluster Ready architecture specification.  Health checks the
  ; functionality of the cluster.  Certification is essentially the
  ; union of compliance and health.
  (slot clck-mode (type SYMBOL)
        (allowed-values certification compliance health)
        (default health))

  ; The maximum allowable age of a data point, in seconds, before a
  ; data point is considered "too old".  The default is 1 week.
  (slot data-age-threshold (type NUMBER) (default 604800))

  ; The number of allowable median absolute deviations allowed before
  ; a dgemm value is considered an outlier.
  (slot dgemm-number-of-mad (type NUMBER) (default 6) (range 0 ?VARIABLE))

  ; The ratio of the measure dgemm performance to the theoretical peak
  ; value must be greater than or equal to this value.
  (slot dgemm-peak-fraction (type NUMBER) (default 0.9) (range 0 1))

  (multislot environment-blacklist (type STRING)
             (default (create$ "")))

  ; The number of allowable median absolute deviations allowed before
  ; a hpl pairwise performance value is considered an outlier.
  (slot hpl-number-of-mad (type NUMBER) (default 6) (range 0 ?VARIABLE))

  (slot imb-pingpong-number-of-mad (type NUMBER) (default 6)
        (range 0 ?VARIABLE))

  (multislot kernel-blacklist (type STRING)
             (default (create$ "")))

  (multislot lshw-blacklist (type STRING)
             (default (create$ "")))

  (slot lshw-uniformity-threshold (type NUMBER) (default 0.9)
        (range 0 1))

  ; The maximum allowable deviation in bytes from the median memory
  ; size before a memory size is considered non-uniform.
  (slot memory-uniformity-threshold (type NUMBER) (default 268435456)
        (range 0 ?VARIABLE))

  (slot ntp-offset-threshold (type NUMBER)
        (default 0.1))

  (slot rpm-uniformity-threshold (type NUMBER) (default 0.9)
        (range 0 1))

)


(defrule default-config
  "Create the [config] instance if one does not already exist."
  (not (object (is-a CONFIG) (name [config])))
  =>
  (make-instance [config] of CONFIG))
