; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defclass FAULT
  "Fundamental base class for out of specification conditions.  This
   class should be inherited by all classes that need the base
   characteristics to describe an out of specification condition, such
   as the confidence and severity of the condition."
  (is-a USER)

  (multislot node_id (type SYMBOL))
  (multislot args)
  (slot confidence (type NUMBER) (default 50.0) (range 0.0 100.0))
  (slot id (type STRING))
  (slot remedy (type STRING))
  (multislot remedy-args)
  (slot severity (type NUMBER) (default 50.0) (range 0.0 100.0))
  (multislot source (type INSTANCE-ADDRESS)) ; instance(s) that are the
                                             ; basis of the FAULT
  (slot suppressed (type SYMBOL) (allowed-values TRUE FALSE) (default FALSE)))

(defmessage-handler FAULT put-source around ($?list)
  "Allow both instance names and instance addresses to be passed to
   put-source and guarantee that an address is ultimately stored."
  (bind ?new (create$ ))
  (progn$ (?i ?list)
          (bind ?new (insert$ ?new (+ (length$ ?new) 1) (instance-address ?i))))
  (override-next-handler ?new))
