; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defclass NETWORK_ADAPTER
  "Base class for physical network adapters."
  (is-a USER)
  (role concrete)

  (slot mpi-dapl-bandwidth (type FLOAT) (default 1.0)) ; units = MB/s
  (slot mpi-dapl-latency (type FLOAT) (default 1e6))   ; units = microseconds
  (slot mpi-ofa-bandwidth (type FLOAT) (default 1.0))  ; units = MB/s
  (slot mpi-ofa-latency (type FLOAT) (default 1e6))    ; units = microseconds
  (slot mpi-tcp-bandwidth (type FLOAT) (default 1.0))  ; units = MB/s
  (slot mpi-tcp-latency (type FLOAT) (default 1e6))    ; units = microseconds
  (slot mpi-tmi-bandwidth (type FLOAT) (default 1.0))  ; units = MB/s
  (slot mpi-tmi-latency (type FLOAT) (default 1e6))    ; units = microseconds
  (slot pci_id (type SYMBOL))
  (slot pci_name (type STRING)))

(defclass ETHERNET_ADAPTER
  "Base class for physical Ethernet adapters."
  (is-a NETWORK_ADAPTER)
  (role concrete)

  (slot speed (type SYMBOL) (allowed-values 1GbE 10GbE 40GbE 100GbE)
        (default 1GbE) (visibility public)))

(defclass INFINIBAND_ADAPTER
  "Base class for physical InfiniBand adapters."
  (is-a NETWORK_ADAPTER)
  (role concrete)

  (slot speed (type SYMBOL) (allowed-values SDR DDR QDR FDR EDR)
        (default QDR) (visibility public)))

(defclass INTEL-ETHERNET_ADAPTER
  "Class for Intel Ethernet adapters."
  (is-a ETHERNET_ADAPTER)
  (role concrete)
  (pattern-match reactive))

(defclass INTEL-INFINIBAND_ADAPTER
  "Class for Intel InfiniBand adapters."
  (is-a INFINIBAND_ADAPTER)
  (role concrete)
  (pattern-match reactive))

(defclass MELLANOX-INFINIBAND_ADAPTER
  "Class for Mellanox InfiniBand adapters."
  (is-a INFINIBAND_ADAPTER)
  (role concrete)
  (pattern-match reactive))
