; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defclass SIGN
  "Abstract class to describe signs discovered by examining
   observations.  Derived from the FAULT class to get confidence,
   severity, etc. slots.  The state slot transitions from initialized,
   when the sign is created, to observed when it is first raised by a
   rule, and finally to diagnosed when it is used to make a higher
   level diagnosis."
  (is-a FAULT)
  (role abstract)
  (pattern-match non-reactive)

  (slot state (type SYMBOL) (allowed-values initialized observed diagnosed)
        (default initialized) (visibility public))
  (slot value))

(defclass BOOLEAN_SIGN
  "Specialized sign class to contain boolean signs."
  (is-a SIGN)
  (role concrete)
  (pattern-match reactive)

  (slot value (type SYMBOL) (allowed-values TRUE FALSE) (default FALSE)
        (source exclusive)))

(defclass COUNTER_SIGN
  "Specialized sign class to contain counter signs."
  (is-a SIGN)
  (role concrete)
  (pattern-match reactive)

  (slot value (type INTEGER) (default 0) (source exclusive)))

(defclass GENERIC_SIGN
  "General sign class to contain arbitrary signs.  Use one of the
   specialized sign classes if possible."
  (is-a SIGN)
  (role concrete)
  (pattern-match reactive))

(defclass PERFORMANCE_SIGN
  "Specialized sign class to contain performance signs."
  (is-a SIGN)
  (role concrete)
  (pattern-match reactive)

  (slot value (type SYMBOL) (allowed-values normal substandard invalid)
        (default normal) (source exclusive)))

(defmessage-handler COUNTER_SIGN increment ()
  "Increment the COUNTER_SIGN value slot by 1."
  (bind ?self:value (+ 1 ?self:value)))
