; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defclass BASE_TIMESTAMP
  "Base class for data point timestamps.  This class should be
   inherited by all classes that need to have a timestamp slot."
  (is-a USER)
  (role abstract)

  ; seconds since epoch
  (slot timestamp (type NUMBER) (default 0) (visibility public)))

(defclass TIMESTAMP
  "This class should be need to contain logic that requires knowledge
  of which of multiple data points is the latest.

  Note: ideally the rule that sets the latest-timestamp slot would be
  defined in this file.  However, as described in section 5.4.1.7 of
  the CLIPS Basic Programming Guide, this rule needs to be defined
  after all the files in classes/ have been loaded."
  (is-a BASE_TIMESTAMP)
  (role abstract)

  (multislot _timestamp_keys (type SYMBOL)
             (pattern-match non-reactive) ; critical for performance
             (storage shared)
             (visibility private))
  (multislot _maximum_timestamp (type NUMBER)
             (pattern-match non-reactive) ; critical for performance
             (storage shared)
             (visibility private))
  (slot latest-timestamp (type SYMBOL) (allowed-values TRUE FALSE)
        (default FALSE)))

(defmessage-handler TIMESTAMP init after ()
  "Keep track of the maximum timestamp."
  (bind ?key (sym-cat (class ?self) - (send ?self get-node_id)))
  (bind ?pos (member$ ?key ?self:_timestamp_keys))
  (if ?pos then
    ; key/value pair already exists
    (bind ?current-max (nth$ ?pos ?self:_maximum_timestamp))
    (if (> ?self:timestamp ?current-max) then
      (slot-replace$ ?self _maximum_timestamp ?pos ?pos ?self:timestamp))
    else
    ; key/value pair needs to be created
    (slot-insert$ ?self _timestamp_keys 1 ?key)
    (slot-insert$ ?self _maximum_timestamp 1 ?self:timestamp)))

(defmessage-handler TIMESTAMP get-maximum-timestamp ()
  "Return the maximum timestamp."
  (bind ?key (sym-cat (class ?self) - (send ?self get-node_id)))
  (bind ?pos (member$ ?key ?self:_timestamp_keys))
  (if ?pos
      then (return (nth$ ?pos ?self:_maximum_timestamp))
      else (return FALSE)))

(defrule set-latest-timestamp
  "Set latest-timestamp to TRUE for those instances that correspond to
   the maximum timestamp."
  (declare (salience ?*first-salience*))
  =>
  (do-for-all-instances ((?i TIMESTAMP))
                        (>= ?i:timestamp (send ?i get-maximum-timestamp))
                        (send ?i put-latest-timestamp TRUE)))
