; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defrule all_to_all-data-missing
  "Check that all-to-all data is available."
  (object (is-a CONFIG) (name [config]) (clck-checks $? all_to_all $?))
  (object (is-a NODE) (node_id ?node))
  (not (object (is-a ALL_TO_ALL) (perspective-node_id ?node)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 90)
                 (state observed) (value TRUE)
                 (id "all_to_all-data-missing")
                 (remedy "all_to_all-data-missing-remedy")))

(defrule ip-address-not-consistent
  "If the ip address of a node differs from the perspective of
   different nodes this rules will fire. The ip address of a
   particular node must be the same on all cluster nodes."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? all_to_all $?))
  ?obj <- (object (is-a ALL_TO_ALL) (node_id ?node)
                  (ip-address ?ip&:(!= (str-length ?ip) 0))
                  (perspective-node_id ?pnode)
                  (multiset_control TRUE))
  (object (is-a NODE) (node_id ?pnode))
  (test (< (send ?obj fraction
                 (sym-cat (send ?obj uniformity-key) ?node) ?ip) 0.9))
  =>
  (bind ?fraction (* 100 (send ?obj fraction
                               (sym-cat (send ?obj uniformity-key) ?node) ?ip)))
  (make-instance of BOOLEAN_SIGN (node_id ?pnode)
                 (confidence (- 100 ?fraction)) (severity 80) (source ?obj)
                 (state observed) (value TRUE) (id "ip-address-not-consistent")
                 (args (create$ ?fraction ?node ?ip))))

(defrule unable-to-obtain-ip-address
  "If hostname -i does not return a valid ip address the connector
   will pass an empty string to the clips slot for the ip address and
   this rule will fire."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? all_to_all $?))
  ?obj <- (object (is-a ALL_TO_ALL) (node_id ?node)
                  (ip-address ?ip) (perspective-node_id ?pnode))
  (object (is-a NODE) (node_id ?pnode))
  (test (= (str-compare ?ip "") 0))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?pnode)
                 (confidence 30) (severity 80) (source ?obj)
                 (state observed) (value TRUE)
                 (id "unable-to-obtain-ip-address")
                 (args (create$ ?node))))
