; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defrule data-is-too-old-initial
  "If there are any signs for out of date data, create a data is too
   old diagnosis and mark the sign as diagnosed.  This rule only fires
   for the first data is too old sign per node, i.e., when the
   diagnosis does not already exist.  Once the diagnosis exists, it
   should not be duplicated.  Thus, there is a corresponding rule,
   data-is-too-old-subsequent, for the case where there are multiple
   signs leading to this diagnosis."
  (object (is-a CONFIG) (name [config])
          (clck-mode certification | compliance | health))
  ?sign <- (object (is-a SIGN) (node_id ?node)
                   (id "heartbeat-is-too-old" |
                       "latest-dgemm-data-is-too-old" |
                       "latest-imb-pingpong-data-is-too-old" |
                       "latest-iozone-is-too-old" |
                       "latest-stream-data-is-too-old")
                   (state ~diagnosed))
  (not (object (is-a DIAGNOSIS) (node_id ?node) (id "data-is-too-old")))
  =>
  (send ?sign put-state diagnosed)
  (make-instance of DIAGNOSIS (node_id ?node)
                 (confidence (send ?sign get-confidence))
                 (severity (send ?sign get-severity))
                 (id "data-is-too-old")
                 (source (send ?sign get-source))
                 (remedy "data-is-too-old-remedy")))

(defrule data-is-too-old-subsequent
  "This rule is related to data-is-too-old-initial.  The difference is
   that this rule fires only after the initial diagnosis has already
   been created.  This rule marks the sign as diagnosed, and also adds
   to the list of signs that produced the diagnosis."
  (object (is-a CONFIG) (name [config])
          (clck-mode certification | compliance | health))
  ?diag <- (object (is-a DIAGNOSIS) (node_id ?node) (id "data-is-too-old"))
  ?sign <- (object (is-a SIGN) (node_id ?node)
                   (id "heartbeat-is-too-old" |
                       "latest-dgemm-data-is-too-old" |
                       "latest-imb-pingpong-data-is-too-old" |
                       "latest-iozone-is-too-old" |
                       "latest-stream-data-is-too-old")
                   (state ~diagnosed))
  =>
  (send ?sign put-state diagnosed)
  (send ?diag put-confidence (combine-confidence (send ?diag get-confidence)
                                                 (send ?sign get-confidence)))
  (send ?diag put-severity (combine-severity (send ?diag get-severity)
                                             (send ?sign get-severity)))
  (slot-insert$ ?diag source 1 (send ?sign get-source)))

(defrule no-data-initial
  "If there are any signs for missing data, create a no data diagnosis
   and mark the sign as diagnosed.  This rule only fires for the first
   no data sign per node, i.e., when the diagnosis does not already
   exist.  Once the diagnosis exists, it should not be duplicated.
   Thus, there is a corresponding rule, no-data-subsequent, for the
   case where there are multiple signs leading to this diagnosis."
  (object (is-a CONFIG) (name [config])
          (clck-mode certification | compliance | health))
  ?sign <- (object (is-a SIGN) (node_id $?node)
                   (id "all_to_all-data-missing" |
                       "cpu-data-missing" |
                       "dgemm-data-missing" |
                       "environment-data-missing" |
                       "ethernet-data-missing" |
                       "hpl-data-missing" |
                       "icr_cluster-data-missing" |
                       "icr_version-data-missing" |
                       "imb_pingpong-data-missing" |
                       "intel_cluster_runtimes-data-missing" |
                       "infiniband-data-missing" |
                       "iozone-data-missing" |
                       "java-data-missing" |
                       "kernel-data-missing" |
                       "kernel_param-data-missing" |
                       "libraries-data-missing" |
                       "lshw-data-missing" |
                       "memory-data-missing" |
                       "miccheck-data-missing" |
                       "micinfo-data-missing" |
                       "mount-data-missing" |
                       "mpi_internode-data-missing" |
                       "mpi_local-data-missing" |
                       "ntp-data-missing" |
                       "offload_phi-data-missing" |
                       "perl-data-missing" |
                       "process-data-missing" |
                       "python-data-missing" |
                       "rpm-data-missing" |
                       "shells-data-missing" |
                       "storage-data-missing" |
                       "stream-data-missing" |
                       "tcl-data-missing" |
                       "x11_tools-data-missing" |
                       "missing-shell" |
                       "no-heartbeat")
                   (state ~diagnosed))
  (not (object (is-a DIAGNOSIS) (node_id $?node) (id "no-data")))
  =>
  (send ?sign put-state diagnosed)
  (make-instance of DIAGNOSIS (node_id ?node)
                 (confidence (send ?sign get-confidence))
                 (severity (send ?sign get-severity))
                 (id "no-data") (source (send ?sign get-source))
                 (remedy "no-data-remedy")))

(defrule no-data-subsequent
  "This rule is related to no-data-initial.  The difference is that
   this rule fires only after the initial diagnosis has already been
   created.  This rule marks the sign as diagnosed, and also adds to
   the list of signs that produced the diagnosis."
  (object (is-a CONFIG) (name [config])
          (clck-mode certification | compliance | health))
  ?diag <- (object (is-a DIAGNOSIS) (node_id $?node) (id "no-data"))
  ?sign <- (object (is-a SIGN) (node_id $?node)
                   (id "all_to_all-data-missing" |
                       "cpu-data-missing" |
                       "dgemm-data-missing" |
                       "environment-data-missing" |
                       "ethernet-data-missing" |
                       "hpl-data-missing" |
                       "icr_cluster-data-missing" |
                       "icr_version-data-missing" |
                       "imb_pingpong-data-missing" |
                       "intel_cluster_runtimes-data-missing" |
                       "infiniband-data-missing" |
                       "iozone-data-missing" |
                       "java-data-missing" |
                       "kernel-data-missing" |
                       "kernel_param-data-missing" |
                       "libraries-data-missing" |
                       "lshw-data-missing" |
                       "memory-data-missing" |
                       "miccheck-data-missing" |
                       "micinfo-data-missing" |
                       "mount-data-missing" |
                       "mpi_internode-data-missing" |
                       "mpi_local-data-missing" |
                       "ntp-data-missing" |
                       "offload_phi-data-missing" |
                       "perl-data-missing" |
                       "process-data-missing" |
                       "python-data-missing" |
                       "rpm-data-missing" |
                       "shells-data-missing" |
                       "storage-data-missing" |
                       "stream-data-missing" |
                       "tcl-data-missing" |
                       "x11_tools-data-missing" |
                       "missing-shell" |
                       "no-heartbeat")
                   (state ~diagnosed))
  =>
  (send ?sign put-state diagnosed)
  (send ?diag put-confidence (combine-confidence (send ?diag get-confidence)
                                                 (send ?sign get-confidence)))
  (send ?diag put-severity (combine-severity (send ?diag get-severity)
                                             (send ?sign get-severity)))
  (slot-insert$ ?diag source 1 (send ?sign get-source)))

(defrule non-uniform-hardware-initial
  "If there are any signs for non-uniform hardware, create a
   non-uniform hardware diagnosis and mark the sign as diagnosed.
   This rule only fires for the first non-uniform hardware sign per
   node, i.e., when the diagnosis does not already exist.  Once the
   diagnosis exists, it should not be duplicated.  Thus, there is a
   corresponding rule, non-uniform-hardware-subsequent, for the case
   where there are multiple signs leading to this diagnosis."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health))
  ?sign <- (object (is-a SIGN) (node_id ?node)
                   (id "cpu-model-name-not-uniform" |
                       "ethernet-firmware-version-is-not-consistent" |
                       "infiniband-ca-type-is-not-consistent" |
                       "infiniband-device-is-not-consistent" |
                       "infiniband-firmware-version-is-not-consistent" |
                       "infiniband-hardware-version-is-not-consistent" |
                       "infiniband-physical-state-is-not-consistent" |
                       "infiniband-physlot-is-not-consistent" |
                       "infiniband-rate-is-not-consistent" |
                       "infiniband-rev-is-not-consistent" |
                       "infiniband-state-is-not-consistent" |
                       "lshw-key-missing" |
                       "lshw-not-uniform" |
                       "memory-not-uniform" |
                       "micinfo-device-flash-not-uniform" |
                       "micinfo-device-smc-boot-loader-not-uniform" |
                       "micinfo-device-smc-firmware-not-uniform" |
                       "micinfo-device-stepping-not-uniform")
                   (state ~diagnosed))
  (not (object (is-a DIAGNOSIS) (node_id ?node) (id "non-uniform-hardware")))
  =>
  (send ?sign put-state diagnosed)
  (make-instance of DIAGNOSIS (node_id ?node)
                 (confidence (send ?sign get-confidence))
                 (severity (send ?sign get-severity))
                 (id "non-uniform-hardware")
                 (source (send ?sign get-source))))

(defrule non-uniform-hardware-subsequent
  "This rule is related to non-uniform-hardware-initial.  The
   difference is that this rule fires only after the initial diagnosis
   has already been created.  This rule marks the sign as diagnosed,
   and also adds to the list of signs that produced the diagnosis."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health))
  ?diag <- (object (is-a DIAGNOSIS) (node_id ?node) (id "non-uniform-hardware"))
  ?sign <- (object (is-a SIGN) (node_id ?node)
                   (id "cpu-model-name-not-uniform" |
                       "ethernet-firmware-version-is-not-consistent" |
                       "infiniband-ca-type-is-not-consistent" |
                       "infiniband-device-is-not-consistent" |
                       "infiniband-firmware-version-is-not-consistent" |
                       "infiniband-hardware-version-is-not-consistent" |
                       "infiniband-physical-state-is-not-consistent" |
                       "infiniband-physlot-is-not-consistent" |
                       "infiniband-rate-is-not-consistent" |
                       "infiniband-rev-is-not-consistent" |
                       "infiniband-state-is-not-consistent" |
                       "lshw-key-missing" |
                       "lshw-not-uniform" |
                       "memory-not-uniform" |
                       "micinfo-device-flash-not-uniform" |
                       "micinfo-device-smc-boot-loader-not-uniform" |
                       "micinfo-device-smc-firmware-not-uniform" |
                       "micinfo-device-stepping-not-uniform")
                   (state ~diagnosed))
  =>
  (send ?sign put-state diagnosed)
  (send ?diag put-confidence (combine-confidence (send ?diag get-confidence)
                                                 (send ?sign get-confidence)))
  (send ?diag put-severity (combine-severity (send ?diag get-severity)
                                             (send ?sign get-severity)))
  (slot-insert$ ?diag source 1 (send ?sign get-source)))

(defrule non-uniform-software-initial
  "If there are any signs for non-uniform software, create a
   non-uniform software diagnosis and mark the sign as diagnosed.
   This rule only fires for the first non-uniform software sign per
   node, i.e., when the diagnosis does not already exist.  Once the
   diagnosis exists, it should not be duplicated.  Thus, there is a
   corresponding rule, non-uniform-software-subsequent, for the case
   where there are multiple signs leading to this diagnosis."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health))
  ?sign <- (object (is-a SIGN) (node_id ?node)
                   (id "environment-variable-not-uniform" |
                       "ethernet-driver-is-not-consistent" |
                       "ethernet-driver-version-is-not-consistent" |
                       "ethernet-mtu-is-not-consistent" |
                       "infiniband-driver-is-not-consistent" |
                       "infiniband-memlock-is-not-consistent" |
                       "infiniband-ofed-version-is-not-consistent" |
                       "ip-address-not-consistent" |
                       "java-not-uniform" |
                       "kernel-not-uniform" |
                       "kernel-param-not-uniform" |
                       "micinfo-device-uos-not-uniform" |
                       "micinfo-host-driver-not-uniform" |
                       "micinfo-host-mpss-not-uniform" |
                       "mount-not-uniform-home-inode" |
                       "mount-not-uniform-home-path" |
                       "mpi-local-path-not-uniform" |
                       "python-not-uniform" |
                       "perl-not-uniform" |
                       "rpm-is-missing" |
                       "rpm-is-extra" |
                       "tcl-not-uniform")
                   (state ~diagnosed))
  (not (object (is-a DIAGNOSIS) (node_id ?node) (id "non-uniform-software")))
  =>
  (send ?sign put-state diagnosed)
  (make-instance of DIAGNOSIS (node_id ?node)
                 (confidence (send ?sign get-confidence))
                 (severity (send ?sign get-severity))
                 (id "non-uniform-software")
                 (source (send ?sign get-source))))

(defrule non-uniform-software-subsequent
  "This rule is related to non-uniform-software-initial.  The
   difference is that this rule fires only after the initial diagnosis
   has already been created.  This rule marks the sign as diagnosed,
   and also adds to the list of signs that produced the diagnosis."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health))
  ?diag <- (object (is-a DIAGNOSIS) (node_id ?node) (id "non-uniform-software"))
  ?sign <- (object (is-a SIGN) (node_id ?node)
                   (id "environment-variable-not-uniform" |
                       "ethernet-driver-is-not-consistent" |
                       "ethernet-driver-version-is-not-consistent" |
                       "ethernet-mtu-is-not-consistent" |
                       "infiniband-driver-is-not-consistent" |
                       "infiniband-memlock-is-not-consistent" |
                       "infiniband-ofed-version-is-not-consistent" |
                       "ip-address-not-consistent" |
                       "java-not-uniform" |
                       "kernel-not-uniform" |
                       "kernel-param-not-uniform" |
                       "micinfo-device-uos-not-uniform" |
                       "micinfo-host-driver-not-uniform" |
                       "micinfo-host-mpss-not-uniform" |
                       "mount-not-uniform-home-inode" |
                       "mount-not-uniform-home-path" |
                       "mpi-local-path-not-uniform" |
                       "python-not-uniform" |
                       "perl-not-uniform" |
                       "rpm-is-missing" |
                       "rpm-is-extra" |
                       "tcl-not-uniform")
                   (state ~diagnosed))
  =>
  (send ?sign put-state diagnosed)
  (send ?diag put-confidence (combine-confidence (send ?diag get-confidence)
                                                 (send ?sign get-confidence)))
  (send ?diag put-severity (combine-severity (send ?diag get-severity)
                                             (send ?sign get-severity)))
  (slot-insert$ ?diag source 1 (send ?sign get-source)))

(defrule not-icr-compliant-initial
  "If there are any signs for Intel(R) Cluster Ready non-compliance,
   create a not-ICR diagnosis and mark the sign as diagnosed.  This
   rule only fires for the first non-compliance sign per node, i.e.,
   when the diagnosis does not already exist.  Once the diagnosis
   exists, it should not be duplicated.  Thus, there is a
   corresponding rule, not-icr-compliant-subsequent, for the case
   where there are multiple signs leading to this diagnosis."
  (object (is-a CONFIG) (name [config]) (clck-mode certification|compliance))
  ?sign <- (object (is-a SIGN) (node_id ?node)
                   (id "cluster-no-headnode" |
                       "cluster-no-min-compute-certification" |
                       "cluster-no-min-compute-compliance" |
                       "cluster-too-enhanced" |
                       "cpu-not-intel64" |
                       "icr-file-not-found" |
                       "icr-file-other-error" |
                       "intel-cluster-runtime-tool-not-compliant" |
                       "intel-cluster-runtime-tool-not-found" |
                       "java-not-found" |
                       "java-not-icr" |
                       "kernel-not-icr" |
                       "latest-icr-version" |
                       "memory-minimum-per-core" |
                       "memory-minimum-required" |
                       "missing-x11-tool" |
                       "mpi-local-path-not-uniform" |
                       "missing-icr-library" |
                       "perl-not-found" |
                       "perl-not-icr" |
                       "processor-not-intel64" |
                       "python-not-found" |
                       "python-not-icr" |
                       "storage-icr-compute" |
                       "storage-icr-head" |
                       "tcl-not-found" |
                       "tcl-not-icr")
                   (state ~diagnosed))
  (not (object (is-a DIAGNOSIS) (node_id ?node) (id "not-icr-compliant")))
  =>
  (send ?sign put-state diagnosed)
  (make-instance of DIAGNOSIS (node_id ?node)
                 (confidence (send ?sign get-confidence))
                 (severity (send ?sign get-severity))
                 (id "not-icr-compliant")
                 (source (send ?sign get-source))))

(defrule not-icr-compliant-subsequent
  "This rule is related to not-icr-compliant-initial.  The difference
   is that this rule fires only after the initial diagnosis has
   already been created.  This rule marks the sign as diagnosed, and
   also adds to the list of signs that produced the diagnosis."
  (object (is-a CONFIG) (name [config]) (clck-mode certification|compliance))
  ?diag <- (object (is-a DIAGNOSIS) (node_id ?node) (id "not-icr-compliant"))
  ?sign <- (object (is-a SIGN) (node_id ?node)
                   (id "cluster-no-headnode" |
                       "cluster-no-min-compute-certification" |
                       "cluster-no-min-compute-compliance" |
                       "cluster-too-enhanced" |
                       "cpu-not-intel64" |
                       "icr-file-not-found" |
                       "icr-file-other-error" |
                       "intel-cluster-runtime-tool-not-compliant" |
                       "intel-cluster-runtime-tool-not-found" |
                       "java-not-found" |
                       "java-not-icr" |
                       "kernel-not-icr" |
                       "latest-icr-version" |
                       "memory-minimum-per-core" |
                       "memory-minimum-required" |
                       "missing-x11-tool" |
                       "mpi-local-path-not-uniform" |
                       "missing-icr-library" |
                       "perl-not-found" |
                       "perl-not-icr" |
                       "processor-not-intel64" |
                       "python-not-found" |
                       "python-not-icr" |
                       "storage-icr-compute" |
                       "storage-icr-head" |
                       "tcl-not-found" |
                       "tcl-not-icr")
                   (state ~diagnosed))
  =>
  (send ?sign put-state diagnosed)
  (send ?diag put-confidence (combine-confidence (send ?diag get-confidence)
                                                 (send ?sign get-confidence)))
  (send ?diag put-severity (combine-severity (send ?diag get-severity)
                                             (send ?sign get-severity)))
  (slot-insert$ ?diag source 1 (send ?sign get-source)))

(defrule substandard-dgemm-due-to-high-cpu-process
  "Diagnose substandard DGEMM performance issues due to a conflicting
   process that is consuming a high amount of cpu.  If the
   dgemm-performance sign is substandard and the high-cpu-process sign
   is true and the associated data points are close together in
   time (within 10 minutes), then conclude the substandard performance
   is due to the high cpu process."
  ?sign1 <- (object (is-a SIGN) (node_id ?node)
                    (id "latest-dgemm-data-is-substandard") (source ?dgemm)
                    (value substandard))
  ?sign2 <- (object (is-a SIGN) (node_id ?node) (id "process-is-high-cpu")
                    (source ?process) (value TRUE))
  (test (< (abs (- (send ?dgemm get-timestamp)
                   (send ?process get-timestamp))) 600))
  =>
  ; the further apart the two data points in time, the lower the confidence
  (bind ?confidence (combine-confidence 
                     (send ?sign1 get-confidence)
                     (send ?sign2 get-confidence)
                     (* (- 1 (/ (abs (- (send ?dgemm get-timestamp)
                                        (send ?process get-timestamp)))
                                600)) 100)))
  (bind ?severity (combine-severity (send ?sign1 get-severity)
                                    (send ?sign2 get-severity)))
  (make-instance of DIAGNOSIS (node_id ?node) (confidence ?confidence)
                 (severity ?severity)
                 (id "substandard-dgemm-due-to-high-cpu-process")
                 (args (create$ (/ (send ?dgemm get-flops) 1e9)
                                (send ?process get-pid)))
                 (remedy "process-is-high-cpu-remedy")
                 (remedy-args (create$ (send ?process get-pid))))
  (modify-instance ?sign1 (state diagnosed))
  (modify-instance ?sign2 (state diagnosed)))

(defrule substandard-dgemm-due-to-high-memory-process
  "Diagnose substandard DGEMM performance issues due to a conflicting
   process that is consuming a large amount of memory.  If the
   dgemm-performance sign is substandard and the high-memory-process
   sign is true and the associated data points are close together in
   time (within 10 minutes), then conclude the substandard performance
   is due to the high memory process."
  ?sign1 <- (object (is-a SIGN) (node_id ?node)
                    (id "latest-dgemm-data-is-substandard") (source ?dgemm)
                    (value substandard))
  ?sign2 <- (object (is-a SIGN) (node_id ?node) (args $?args)
                    (id "process-is-high-memory") (source ?process)
                    (value TRUE))
  (test (< (abs (- (send ?dgemm get-timestamp)
                   (send ?process get-timestamp))) 600))
  =>
  ; the further apart the two data points in time, the lower the confidence
  (bind ?confidence (combine-confidence
                     (send ?sign1 get-confidence)
                     (send ?sign2 get-confidence)
                     (* (- 1 (/ (abs (- (send ?dgemm get-timestamp)
                                        (send ?process get-timestamp)))
                                600)) 100)))
  (bind ?severity (combine-severity (send ?sign1 get-severity)
                                    (send ?sign2 get-severity)))
  (make-instance of DIAGNOSIS (node_id ?node) (confidence ?confidence)
                 (severity ?severity)
                 (id "substandard-dgemm-due-to-high-memory-process")
                 (args (create$ (/ (send ?dgemm get-flops) 1e9)
                                (send ?process get-pid)))
                 (remedy "process-is-high-memory-remedy")
                 (remedy-args (create$ (send ?process get-pid))))
  (modify-instance ?sign1 (state diagnosed))
  (modify-instance ?sign2 (state diagnosed)))

(defrule substandard-imb-pingpong-latency-due-to-ethernet-coalescing
  "Diagnose substandard IMB pingpong latency performance issues due to
   Ethernet interrupt coalescing not being disabled.  If the
   imb-pingpong-latency-threshold sign is TRUE (substandard), the
   Intel(R) MPI Library settings are configured to use Ethernet, and
   the ethernet-interrupt-coalescing-is-enabled sign is TRUE, then
   conclude the substandard performance is due to Ethernet interrupt
   coalescing not being disabled.  Note that the Ethernet interrupt
   coalescing only affects IMB pingpong latency, not bandwidth, so
   there is no corresponding rule for bandwidth."

  ;TODO: update to be like the outlier rule
  ?lat-sign <- (object (is-a SIGN)
                       (id "imb-pingpong-latency-threshold")
                       (source ?pair) (value TRUE))
  ?eth-sign <- (object (is-a SIGN)
                       (id "ethernet-interrupt-coalescing-is-enabled")
                       (source ?node) (value TRUE))
  (object (is-a IMB_PINGPONG) (i_mpi_fabrics ?f&:(str-index "tcp" ?f))
          (i_mpi_tcp_netmask ?m&:(str-index "eth" ?m))
          (name ?n&:(eq ?n (instance-name ?pair))))
  =>
  (bind ?confidence (combine-confidence (send ?eth-sign get-confidence)
                                        (send ?lat-sign get-confidence)))
  (bind ?severity (combine-severity (send ?eth-sign get-severity)
                                    (send ?lat-sign get-severity)))
  (make-instance of DIAGNOSIS (node_id (send ?pair get-node_id))
                 (confidence ?confidence) (severity ?severity)
                 (id "substandard-imb-pingpong-latency-due-to-ethernet-coalescing")
                 (args (create$ (send ?pair get-latency)
                                (send ?pair get-i_mpi_tcp_netmask)
                                (send ?pair get-impi-setting-string)))
                 (remedy "substandard-imb-pingpong-latency-due-to-ethernet-coalescing-remedy")
                 (remedy-args (create$ (send ?node get-interface))))
  (modify-instance ?eth-sign (state diagnosed))
  (modify-instance ?lat-sign (state diagnosed)))

(defrule outlier-imb-pingpong-latency-due-to-ethernet-coalescing
  "Diagnose IMB pingpong latency performance outlier issues due to
   Ethernet interrupt coalescing not being disabled.  If the
   imb-pingpong-latency-outlier sign is TRUE, the
   Intel(R) MPI Library settings are configured to use Ethernet, and
   the ethernet-interrupt-coalescing-is-enabled sign is TRUE, then
   conclude the inconsistent performance is due to Ethernet interrupt
   coalescing not being disabled.  Note that the Ethernet interrupt
   coalescing only affects IMB pingpong latency, not bandwidth, so
   there is no corresponding rule for bandwidth."
  ?eth-sign <- (object (is-a SIGN) (node_id ?n)
                       (id "ethernet-interrupt-coalescing-is-enabled")
                       (source ?node) (value TRUE))
  ?lat-sign <- (object (is-a SIGN) (node_id $? ?n $?)
                       (id "imb-pingpong-latency-outlier")
                       (source ?pair) (value TRUE))
  (object (is-a IMB_PINGPONG) (i_mpi_fabrics ?f&:(str-index "tcp" ?f))
          (i_mpi_tcp_netmask ?m&:(str-index "eth" ?m))
          (name ?name&:(eq ?name (instance-name ?pair))))
  (not (object (is-a DIAGNOSIS) (node_id ?n)
               (id "outlier-imb-pingpong-latency-due-to-ethernet-coalescing")))
  =>
  (bind ?confidence (combine-confidence (send ?eth-sign get-confidence)
                                        (send ?lat-sign get-confidence)))
  (bind ?severity (combine-severity (send ?eth-sign get-severity)
                                    (send ?lat-sign get-severity)))
  (make-instance of DIAGNOSIS (node_id ?n)
                 (confidence ?confidence) (severity ?severity)
                 (id "outlier-imb-pingpong-latency-due-to-ethernet-coalescing")
                 (args (create$ (send ?node get-interface)))
                 (remedy "outlier-imb-pingpong-latency-due-to-ethernet-coalescing-remedy")
                 (remedy-args (create$ (send ?node get-interface))))
  (modify-instance ?eth-sign (state diagnosed))
  (modify-instance ?lat-sign (state diagnosed)))

(defrule xeon-phi-coprocessor-not-functional-initial
  "If there are any signs for non-functional Intel(R) Xeon Phi(TM)
   coprocessors, create a non-functional Intel(R) Xeon Phi(TM)
   coprocessor diagnosis and mark the sign as diagnosed.  This rule
   only fires for the first non-functional sign per node, i.e., when
   the diagnosis does not already exist.  Once the diagnosis exists,
   it should not be duplicated.  Thus, there is a corresponding rule,
   xeon-phi-not-funciontal-subsequent, for the case where there are
   multiple signs leading to this diagnosis."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health))
  ?sign <- (object (is-a SIGN) (node_id ?node)
                   (id "miccheck-device-daemon-available-failure" |
                       "miccheck-device-flash-version-failure" |
                       "miccheck-device-online-failure" |
                       "miccheck-host-driver-load-failure" |
                       "miccheck-host-mpssd-run-failure" |
                       "miccheck-host-num-dev-driver-failure" |
                       "miccheck-host-num-dev-os-failure" |
                       "offload-failure")
                   (state ~diagnosed))
  (not (object (is-a DIAGNOSIS) (node_id ?node)
               (id "xeon-phi-coprocessor-not-functional")))
  =>
  (send ?sign put-state diagnosed)
  (make-instance of DIAGNOSIS (node_id ?node)
                 (confidence (send ?sign get-confidence))
                 (severity (send ?sign get-severity))
                 (id "xeon-phi-coprocessor-not-functional")
                 (source (send ?sign get-source))))

(defrule xeon-phi-coprocessor-not-functional-subsequent
  "This rule is related to xeon-phi-coprocessor-not-functional-initial.
   The difference is that this rule fires only after the initial
   diagnosis has already been created.  This rule marks the sign as
   diagnosed, and also adds to the list of signs that produced the
   diagnosis."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health))
  ?diag <- (object (is-a DIAGNOSIS) (node_id ?node)
                   (id "xeon-phi-coprocessor-not-functional"))
  ?sign <- (object (is-a SIGN) (node_id ?node)
                   (id "miccheck-device-daemon-available-failure" |
                       "miccheck-device-flash-version-failure" |
                       "miccheck-device-online-failure" |
                       "miccheck-host-driver-load-failure" |
                       "miccheck-host-mpssd-run-failure" |
                       "miccheck-host-num-dev-driver-failure" |
                       "miccheck-host-num-dev-os-failure" |
                       "offload-failure")
                   (state ~diagnosed))
  =>
  (send ?sign put-state diagnosed)
  (send ?diag put-confidence (combine-confidence (send ?diag get-confidence)
                                                 (send ?sign get-confidence)))
  (send ?diag put-severity (combine-severity (send ?diag get-severity)
                                             (send ?sign get-severity)))
  (slot-insert$ ?diag source 1 (send ?sign get-source)))
