; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defrule ethernet-data-missing
  "Check that ethernet data is available."
  (object (is-a CONFIG) (name [config]) (clck-checks $? ethernet $?))
  (object (is-a NODE) (node_id ?node) (architecture x86_64)
          (role $?role&:(member$ compute ?role)|:(member$ enhanced ?role)))
  (not (object (is-a ETHERNET) (node_id ?node)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 90)
                 (state observed) (value TRUE)
                 (id "ethernet-data-missing")
                 (remedy "ethernet-data-missing-remedy")))

(defrule ethernet-interrupt-coalescing-is-enabled
  "Identify nodes where Ethernet interrupt coalescing is not disabled,
   i.e., rx-usecs is not 0 or 1.  This only matters when using
   Ethernet as the MPI message fabric.  Since the same node may be in
   multiple IMB pingpong pairs, check to see if the sign has already
   been created to avoid duplicates."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? ethernet $?))
  (object (is-a IMB_PINGPONG) (node_id $? ?node $?)
          (i_mpi_fabrics shm:tcp | tcp) (i_mpi_tcp_netmask ?i))
  ?obj <- (object (is-a ETHERNET) (node_id ?node) (interface ?i)
                  (rx-usecs ~0 & ~1))
  (object (is-a NODE) (node_id ?node))
  (not (object (is-a SIGN) (id "ethernet-interrupt-coalescing-is-enabled")
               (source ?obj)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node) (confidence 90) (severity 75)
                 (state observed) (source ?obj) (value TRUE)
                 (id "ethernet-interrupt-coalescing-is-enabled")
                 (args ?i)
                 (remedy "ethernet-interrupt-coalescing-is-enabled-remedy")
                 (remedy-args ?i)))

(defrule ethernet-driver-is-not-consistent
  "Identify inconsistent Ethernet drivers."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? ethernet $?))
  ?obj <- (object (is-a ETHERNET) (driver ?d) (multiset_control TRUE)
                  (node_id ?node))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj uniformity-key driver) ?d) 0.9))
  =>
  (bind ?fraction (- 1 (send ?obj fraction
                             (send ?obj uniformity-key driver) ?d)))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence (* 100 ?fraction)) (severity 80)
                 (state observed) (source ?obj) (value TRUE)
                 (id "ethernet-driver-is-not-consistent")
                 (args (create$
                        (* 100 (send ?obj fraction
                                     (send ?obj uniformity-key driver) ?d))
                        ?d))))

(defrule ethernet-driver-version-is-not-consistent
  "Identify inconsistent Ethernet driver versions."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? ethernet $?))
  ?obj <- (object (is-a ETHERNET) (multiset_control TRUE) (version ?v)
                  (node_id ?node))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj uniformity-key version) ?v) 0.9))
  =>
  (bind ?fraction (- 1 (send ?obj fraction
                             (send ?obj uniformity-key version) ?v)))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence (* 100 ?fraction)) (severity 20)
                 (state observed) (source ?obj) (value TRUE)
                 (id "ethernet-driver-version-is-not-consistent")
                 (args (create$
                        (* 100 (send ?obj fraction
                                     (send ?obj uniformity-key version) ?v))
                        ?v))))

(defrule ethernet-firmware-version-is-not-consistent
  "Identify inconsistent Ethernet firmware versions."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? ethernet $?))
  ?obj <- (object (is-a ETHERNET) (firmware-version ?v) (multiset_control TRUE)
                  (node_id ?node))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj uniformity-key firmware-version) ?v)
           0.9))
  =>
  (bind ?fraction (- 1 (send ?obj fraction
                             (send ?obj uniformity-key firmware-version) ?v)))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence (* 100 ?fraction)) (severity 20)
                 (state observed) (source ?obj) (value TRUE)
                 (id "ethernet-firmware-version-is-not-consistent")
                 (args (create$
                        (* 100 (send ?obj fraction
                                     (send ?obj uniformity-key firmware-version)
                                     ?v))
                        ?v))))

(defrule ethernet-mtu-is-not-consistent
  "Identify inconsistent Ethernet firmware versions."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? ethernet $?))
  ?obj <- (object (is-a ETHERNET) (mtu ?mtu) (multiset_control TRUE)
                  (node_id ?node))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj uniformity-key mtu) ?mtu) 0.9))
  =>
  (bind ?fraction (- 1 (send ?obj fraction
                             (send ?obj uniformity-key mtu) ?mtu)))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence (* 100 ?fraction)) (severity 80)
                 (state observed) (source ?obj) (value TRUE)
                 (id "ethernet-mtu-is-not-consistent")
                 (args (create$
                        (* 100 (send ?obj fraction
                                     (send ?obj uniformity-key mtu) ?mtu))
                        ?mtu))))
