; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defrule hpl-data-missing
  "Check that hpl data is available."
  (object (is-a CONFIG) (name [config]) (clck-checks $? hpl $?))
  (object (is-a NODE) (node_id ?node) (architecture x86_64)
          (role $?role&:(member$ compute ?role)|:(member$ enhanced ?role)))
  (not (object (is-a HPL) (node_id $?nodes&:(member$ ?node ?nodes))))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 90)
                 (state observed) (value TRUE)
                 (id "hpl-data-missing")
                 (remedy "hpl-data-missing-remedy")))

(defrule hpl-cluster-failed
  "Look for cases where HPL cluster ran but there was no 'success' in
   the output."
  (object (is-a CONFIG) (name [config]) (clck-mode health | certification)
          (clck-checks $? hpl $?))
  ?obj <- (object (is-a HPL) (provider CLUSTER) (residual ~PASSED))
  =>
  (make-instance of BOOLEAN_SIGN (node_id (send ?obj get-node_id))
                 (confidence 90) (severity 90)
                 (state observed) (source ?obj) (value TRUE)
                 (id "hpl-cluster-failed")))

(defrule hpl-pairwise-failed
  "Look for cases where HPL pairwise ran but there was no 'success' in
   the output."
  (object (is-a CONFIG) (name [config]) (clck-mode health | certification)
          (clck-checks $? hpl $?))
  ?obj <- (object (is-a HPL) (provider PAIRWISE) (residual ~PASSED))
  =>
  (make-instance of BOOLEAN_SIGN (node_id (send ?obj get-node_id))
                 (confidence 90) (severity 90)
                 (state observed) (source ?obj) (value TRUE)
                 (id "hpl-pairwise-failed")))

(defrule hpl-pairwise-outlier
  "Locate values that are outliers.  An outlier is a value that is
   outside the range defined by the median +/- 6 * median absolute
   deviations.  The statistics are computed using all samples on nodes
   in the same grouping (i.e., have the same HPL statistics key).

   Note: the statistics_control condition is required to ensure that
   all samples are included when computing the statistics."
  (object (is-a CONFIG) (name [config]) (clck-mode health | certification)
          (clck-checks $? hpl $?) (hpl-number-of-mad ?num-mad))
  ?o <- (object (is-a HPL) (provider PAIRWISE) (residual PASSED)
                (gflops ?GFLOPS) (statistics_control TRUE))
  (test (or
         (> ?GFLOPS (+ (send ?o median (send ?o statistics-key))
                       (* ?num-mad (send ?o mad (send ?o statistics-key)))))
         (< ?GFLOPS (- (send ?o median (send ?o statistics-key))
                       (* ?num-mad (send ?o mad (send ?o statistics-key)))))))
  =>
  (bind ?key (send ?o statistics-key))
  (bind ?severity (* (- 1 (gaussian ?GFLOPS
                                    (send ?o median ?key)
                                    (* 3 1.4826 (send ?o mad ?key))))
                     80))
  (make-instance of BOOLEAN_SIGN (node_id (send ?o get-node_id))
                 (confidence 75) (severity ?severity)
                 (state observed) (source ?o)
                 (id "hpl-pairwise-outlier")
                 (args (create$ ?GFLOPS (send ?o median ?key)
                                (* ?num-mad (send ?o mad ?key))
                                (send ?o get-impi-setting-string)))))
