; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defrule icr_cluster-data-missing
  "Check that Intel(R) Cluster Ready cluster data is available."
  (object (is-a CONFIG) (name [config]) (clck-checks $? icr_cluster $?))
  (object (is-a NODE) (node_id ?node))
  (not (object (is-a ICR_CLUSTER) (node_id $? ?node $?)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 90)
                 (state observed) (value TRUE)
                 (id "icr_cluster-data-missing")
                 (remedy "icr_cluster-data-missing-remedy")))

(defrule cluster-no-min-compute-certification
  "Determine whether the cluster has at least 4 compute nodes in
   certification mode"
  (object (is-a CONFIG) (name [config]) (clck-mode certification)
          (clck-checks $? icr_cluster $?))
  ?obj <- (object (is-a ICR_CLUSTER) (architecture x86_64) (role compute)
                  (count ?c&:(< ?c 4)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id (send ?obj get-node_id))
                 (confidence 90) (severity 50) (source ?obj)
                 (state observed) (value TRUE)
                 (id "cluster-no-min-compute-certification")))

(defrule cluster-no-min-compute-compliance
  "Determine whether the cluster has at least 2 compute nodes in
   compliance mode"
  (object (is-a CONFIG) (name [config]) (clck-mode compliance)
          (clck-checks $? icr_cluster $?))
  ?obj <- (object (is-a ICR_CLUSTER) (architecture x86_64) (role compute)
                  (count ?c&:(< ?c 2)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id (send ?obj get-node_id))
                 (confidence 90) (severity 50) (source ?obj)
                 (state observed) (value TRUE)
                 (id "cluster-no-min-compute-compliance")))

(defrule cluster-no-headnode
  "Check whether the cluster has at least 1 head node."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? icr_cluster $?))
  (not (object (is-a NODE) (role $?role&:(member$ head ?role))))
  =>
  (make-instance of BOOLEAN_SIGN (node_id "")
                 (confidence 90) (severity 80)
                 (state observed) (value TRUE) (id "cluster-no-headnode")))

(defrule cluster-too-enhanced
  "Determine whether the cluster has more than the allowed number of
   3 enhanced nodes according to ICR"
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? icr_cluster $?))
  ?obj <- (object (is-a ICR_CLUSTER) (architecture x86_64) (role enhanced)
                  (count ?c&:(> ?c 3)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id (send ?obj get-node_id))
                 (confidence 90) (severity 80) (source ?obj)
                 (state observed) (value TRUE) (id "cluster-too-enhanced")))
