; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defrule intel_cluster_runtimes-data-missing
  "Check that Intel(R) Cluster Runtimes data is available."
  (object (is-a CONFIG) (name [config])
          (clck-checks $? intel_cluster_runtimes $?))
  (object (is-a NODE) (node_id ?node) (architecture x86_64))
  (not (object (is-a INTEL_CLUSTER_RUNTIME_tool) (node_id ?node)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 90)
                 (state observed) (value TRUE)
                 (id "intel_cluster_runtimes-data-missing")
                 (remedy "intel_cluster_runtimes-data-missing-remedy")))

(defrule intel-cluster-runtime-cc-not-compliant
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? intel_cluster_runtimes $?))
  ?obj <- (object (is-a INTEL_CLUSTER_RUNTIME_tool)
                  (node_id ?hostname)
                  (toolname cc )
                  (versions $?ver&:(>(length$ ?ver) 0 ))
                  (latest-timestamp TRUE))
  (object (is-a NODE) (node_id ?hostname) (architecture x86_64))
  (test (< (vercmp (send ?obj calc_maximum_version) "14.0") 0))
  =>
  (bind ?maxver (send ?obj calc_maximum_version))
  (make-instance of BOOLEAN_SIGN (node_id ?hostname)
                 (confidence 90) (severity 80)
                 (state observed) (value TRUE) (source ?obj)
                 (id "intel-cluster-runtime-tool-not-compliant")
                 (args (create$ "Intel(R) C++ Compiler" ?maxver "/opt/intel/cc"
                                "1.3.1" "14.0"))))

(defrule intel-cluster-runtime-cc-not-found
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? intel_cluster_runtimes $?))
  ?obj <- (object (is-a INTEL_CLUSTER_RUNTIME_tool)
                  (node_id ?hostname)
                  (toolname cc)
                  (versions $?ver&:(=(length$ ?ver) 0 ))
                  (latest-timestamp TRUE))
  (object (is-a NODE) (node_id ?hostname) (architecture x86_64))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?hostname)
                 (confidence 90) (severity 80)
                 (state observed) (value TRUE) (source ?obj)
                 (id "intel-cluster-runtime-tool-not-found")
                 (args (create$ "Intel(R) C++ Compiler" "/opt/intel/cc"))))

(defrule intel-cluster-runtime-cce-not-compliant
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? intel_cluster_runtimes $?))
  ?obj <- (object (is-a INTEL_CLUSTER_RUNTIME_tool)
                  (node_id ?hostname)
                  (toolname cce)
                  (versions $?ver&:(>(length$ ?ver) 0 ))
                  (latest-timestamp TRUE))
  (object (is-a NODE) (node_id ?hostname) (architecture x86_64))
  (test (< (vercmp (send ?obj calc_maximum_version) "14.0") 0))
  =>
  (bind ?maxver (send ?obj calc_maximum_version))
  (make-instance of BOOLEAN_SIGN (node_id ?hostname)
                 (confidence 90) (severity 80)
                 (state observed) (value TRUE) (source ?obj)
                 (id "intel-cluster-runtime-tool-not-compliant")
                 (args (create$ "Intel(R) C++ Compiler" ?maxver "/opt/intel/cce"
                                "1.3.1" "14.0"))))

(defrule intel-cluster-runtime-cce-not-found
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? intel_cluster_runtimes $?))
  ?obj <- (object (is-a INTEL_CLUSTER_RUNTIME_tool)
                  (node_id ?hostname)
                  (toolname cce)
                  (versions $?ver&:(=(length$ ?ver) 0 ))
                  (latest-timestamp TRUE))
  (object (is-a NODE) (node_id ?hostname) (architecture x86_64))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?hostname)
                 (confidence 90) (severity 80)
                 (state observed) (value TRUE) (source ?obj)
                 (id "intel-cluster-runtime-tool-not-found")
                 (args (create$ "Intel(R) C++ Compiler" "/opt/intel/cce"))))

(defrule icr-runtime-fc-not-compliant
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? intel_cluster_runtimes $?))
  ?obj <- (object (is-a INTEL_CLUSTER_RUNTIME_tool)
                  (node_id ?hostname)
                  (toolname fc)
                  (versions $?ver&:(>(length$ ?ver) 0 ))
                  (latest-timestamp TRUE))
  (object (is-a NODE) (node_id ?hostname) (architecture x86_64))
  (test (< (vercmp (send ?obj calc_maximum_version) "14.0") 0))
  =>
  (bind ?maxver (send ?obj calc_maximum_version))
  (make-instance of BOOLEAN_SIGN (node_id ?hostname)
                 (confidence 90) (severity 80)
                 (state observed) (value TRUE) (source ?obj)
                 (id "intel-cluster-runtime-tool-not-compliant")
                 (args (create$ "Intel(R) Fortran Compiler" ?maxver
                                "/opt/intel/fc" "1.3.1" "14.0"))))

(defrule icr-runtime-fc-not-found
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? intel_cluster_runtimes $?))
  ?obj <- (object (is-a INTEL_CLUSTER_RUNTIME_tool)
                  (node_id ?hostname)
                  (toolname fc)
                  (versions $?ver&:(=(length$ ?ver) 0 ))
                  (latest-timestamp TRUE))
  (object (is-a NODE) (node_id ?hostname) (architecture x86_64))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?hostname)
                 (confidence 90) (severity 80)
                 (state observed) (value TRUE) (source ?obj)
                 (id "intel-cluster-runtime-tool-not-found")
                 (args (create$ "Intel(R) Fortran Compiler" "/opt/intel/fc"))))

(defrule intel-cluster-runtime-fce-not-compliant
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? intel_cluster_runtimes $?))
  ?obj <- (object (is-a INTEL_CLUSTER_RUNTIME_tool)
                  (node_id ?hostname)
                  (toolname fce)
                  (versions $?ver&:(>(length$ ?ver) 0 ))
                  (latest-timestamp TRUE))
  (object (is-a NODE) (node_id ?hostname) (architecture x86_64))
  (test (< (vercmp (send ?obj calc_maximum_version) "14.0") 0))
  =>
  (bind ?maxver (send ?obj calc_maximum_version))
  (make-instance of BOOLEAN_SIGN (node_id ?hostname)
                 (confidence 90) (severity 80)
                 (state observed) (value TRUE) (source ?obj)
                 (id "intel-cluster-runtime-tool-not-compliant")
                 (args (create$ "Intel(R) Fortran Compiler" ?maxver
                                "/opt/intel/fce" "1.3.1" "14.0"))))

(defrule intel-cluster-runtime-fce-not-found
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? intel_cluster_runtimes $?))
  ?obj <- (object (is-a INTEL_CLUSTER_RUNTIME_tool)
                  (node_id ?hostname)
                  (toolname fce)
                  (versions $?ver&:(=(length$ ?ver) 0 ))
                  (latest-timestamp TRUE))
  (object (is-a NODE) (node_id ?hostname) (architecture x86_64))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?hostname)
                 (confidence 90) (severity 80)
                 (state observed) (value TRUE) (source ?obj)
                 (id "intel-cluster-runtime-tool-not-found")
                 (args (create$ "Intel(R) Fortran Compiler" "/opt/intel/fce"))))

(defrule intel-cluster-runtime-impi-not-compliant
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? intel_cluster_runtimes $?))
  ?obj <- (object (is-a INTEL_CLUSTER_RUNTIME_tool)
                  (node_id ?hostname)
                  (toolname impi)
                  (versions $?ver&:(>(length$ ?ver) 0 ))
                  (latest-timestamp TRUE))
  (object (is-a NODE) (node_id ?hostname) (architecture x86_64))
  (test (< (vercmp (send ?obj calc_maximum_version) "4.1") 0))
  =>
  (bind ?maxver (send ?obj calc_maximum_version))
  (make-instance of BOOLEAN_SIGN (node_id ?hostname)
                 (confidence 90) (severity 80)
                 (state observed) (value TRUE) (source ?obj)
                 (id "intel-cluster-runtime-tool-not-compliant")
                 (args (create$ "Intel(R) MPI Library" ?maxver "/opt/intel/impi"
                                "1.3.1" "4.1"))))

(defrule intel-cluster-runtime-impi-not-found
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? intel_cluster_runtimes $?))
  ?obj <- (object (is-a INTEL_CLUSTER_RUNTIME_tool)
                  (node_id ?hostname)
                  (toolname impi)
                  (versions $?ver&:(=(length$ ?ver) 0 ))
                  (latest-timestamp TRUE))
  (object (is-a NODE) (node_id ?hostname) (architecture x86_64))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?hostname)
                 (confidence 90) (severity 80)
                 (state observed) (value TRUE) (source ?obj)
                 (id "intel-cluster-runtime-tool-not-found")
                 (args (create$ "Intel(R) MPI Library" "/opt/intel/impi"))))

(defrule intel-cluster-runtime-cmkl-not-compliant
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? intel_cluster_runtimes $?))
  ?obj <- (object (is-a INTEL_CLUSTER_RUNTIME_tool)
                  (node_id ?hostname)
                  (toolname cmkl)
                  (versions $?ver&:(>(length$ ?ver) 0 ))
                  (latest-timestamp TRUE))
  (object (is-a NODE) (node_id ?hostname) (architecture x86_64))
  (test (< (vercmp (send ?obj calc_maximum_version) "11.0.1") 0))
  =>
  (bind ?maxver (send ?obj calc_maximum_version))
  (make-instance of BOOLEAN_SIGN (node_id ?hostname)
                 (confidence 90) (severity 80)
                 (state observed) (value TRUE) (source ?obj)
                 (id "intel-cluster-runtime-tool-not-compliant")
                 (args (create$ "Intel(R) Math Kernel Library" ?maxver
                                "/opt/intel/cmkl" "1.3.1" "11.0.1"))))

(defrule intel-cluster-runtime-cmkl-not-found
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? intel_cluster_runtimes $?))
  ?obj <- (object (is-a INTEL_CLUSTER_RUNTIME_tool)
                  (node_id ?hostname)
                  (toolname cmkl)
                  (versions $?ver&:(=(length$ ?ver) 0 ))
                  (latest-timestamp TRUE))
  (object (is-a NODE) (node_id ?hostname) (architecture x86_64))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?hostname)
                 (confidence 90) (severity 80)
                 (state observed) (value TRUE) (source ?obj)
                 (id "intel-cluster-runtime-tool-not-found")
                 (args (create$ "Intel(R) Math Kernel Library"
                                "/opt/intel/cmkl"))))

(defrule intel-cluster-runtime-tbb-not-compliant
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? intel_cluster_runtimes $?))
  ?obj <- (object (is-a INTEL_CLUSTER_RUNTIME_tool)
                  (node_id ?hostname)
                  (toolname tbb)
                  (versions $?ver&:(>(length$ ?ver) 0 ))
                  (latest-timestamp TRUE))
  (object (is-a NODE) (node_id ?hostname) (architecture x86_64))
  (test (< (vercmp (send ?obj calc_maximum_version) "4.1.1") 0))
  =>
  (bind ?maxver (send ?obj calc_maximum_version))
  (make-instance of BOOLEAN_SIGN (node_id ?hostname)
                 (confidence 90) (severity 80)
                 (state observed) (value TRUE) (source ?obj)
                 (id "intel-cluster-runtime-tool-not-compliant")
                 (args (create$ "Intel(R) Threading Building Blocks" ?maxver
                                "/opt/intel/tbb" "1.3.1" "4.1.1"))))

(defrule intel-cluster-runtime-tbb-not-found
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? intel_cluster_runtimes $?))
  ?obj <- (object (is-a INTEL_CLUSTER_RUNTIME_tool)
                  (node_id ?hostname)
                  (toolname tbb)
                  (versions $?ver&:(=(length$ ?ver) 0 ))
                  (latest-timestamp TRUE))
  (object (is-a NODE) (node_id ?hostname) (architecture x86_64))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?hostname)
                 (confidence 90) (severity 80)
                 (state observed) (value TRUE) (source ?obj)
                 (id "intel-cluster-runtime-tool-not-found")
                 (args (create$ "Intel(R) Threading Building Blocks"
                                "/opt/intel/tbb"))))
