; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defrule icr_version-data-missing
  "Check that Intel(R) Cluster Ready version data is available."
  (object (is-a CONFIG) (name [config]) (clck-checks $? icr_version $?))
  (object (is-a NODE) (node_id ?node) (architecture x86_64))
  (not (object (is-a ICR) (node_id ?node)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 90)
                 (state observed) (value TRUE)
                 (id "icr_version-data-missing")
                 (remedy "icr_version-data-missing-remedy")))

(defrule latest-icr-version
  "Determine whether the self-identified Intel(R) Cluster Ready
   version contains the latest version (currently, version 1.3.1)."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? icr_version $?))
  ?obj <- (object (is-a ICR) (node_id ?node) (icr-version $?icr-version)
                  (latest-timestamp TRUE))
  (object (is-a NODE) (node_id ?node) (architecture x86_64))
  (test (and (!= (length$ ?icr-version) 0)
             (not (member$ "1.3.1" ?icr-version))))
  =>
  (if (member$ "1.3" ?icr-version) then (bind ?severity 25)
      else (if (member$ "1.2" ?icr-version) then (bind ?severity 50)
               else (if (member$ "1.1" ?icr-version) then (bind ?severity 75)
                        else (bind ?severity 80))))

  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity ?severity) (source ?obj)
                 (state observed) (value FALSE)
                 (id "latest-icr-version")
                 (args (create$ (implode$ ?icr-version) "1.3.1"))))

(defrule icr-file-not-found
  "If no Intel(R) Cluster Ready versions are found and stderr contains
   the string 'No such file or directory', then the file is missing."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? icr_version $?))
  ?obj <- (object (is-a ICR) (node_id ?node) (icr-version $?icr-version)
                  (stderr ?err) (latest-timestamp TRUE))
  (object (is-a NODE) (node_id ?node) (architecture x86_64))
  (test (and (= (length$ ?icr-version) 0)
             (str-index "No such file or directory" ?err)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 80) (source ?obj)
                 (state observed) (value TRUE) (id "icr-file-not-found")))

(defrule icr-file-other-error
  "If no Intel(R) Cluster Ready versions are found or stderr is not
   empty, then the file may not be readable.  If a version is present
   and stderr is not empty, use lower confidence and severity values,
   since the stderr output may be unrelated.  If no version is present
   and stderr is not empty, then the file is definitely not readable,
   so use high confidence and severity values.  Avoid matching the 'No
   such file or directory' case that is handled separately."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? icr_version $?))
  ?obj <- (object (is-a ICR) (node_id ?node) (icr-version $?icr-version)
                  (stderr ?err) (latest-timestamp TRUE))
  (object (is-a NODE) (node_id ?node) (architecture x86_64))
  (not (object (is-a SIGN) (id "icr-file-not-found") (source ?obj)))
  (test (or (= (length$ ?icr-version) 0) (!= (str-length ?err) 0)))
  =>
  (if (> (length$ ?icr-version) 0)
      then (bind ?confidence 25)  (bind ?severity 25)
      else (bind ?confidence 80) (bind ?severity 80))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence ?confidence) (severity ?severity) (source ?obj)
                 (state observed) (value TRUE) (id "icr-file-other-error")
                 (args ?err)))
