; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defrule imb_pingpong-data-missing
  "Check that IMB PingPong data is available."
  (object (is-a CONFIG) (name [config]) (clck-checks $? imb_pingpong $?))
  (object (is-a NODE) (node_id ?node)
          (role $?role&:(member$ compute ?role)|:(member$ enhanced ?role)))
  (not (object (is-a IMB_PINGPONG) (node_id $? ?node $?)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 90)
                 (state observed) (value TRUE)
                 (id "imb_pingpong-data-missing")
                 (remedy "imb_pingpong-data-missing-remedy")))

(defrule imb-pingpong-bandwidth-threshold
  "Check that the measured Intel(R) MPI Benchmark PingPong bandwidth
   is greater than or equal to the expected bandwidth."
  (object (name [config]) (clck-mode certification | health)
          (clck-checks $? imb_pingpong $?))
  ?obj <- (object (is-a IMB_PINGPONG) (node_id ?node0 ?node1)
                  (bandwidth ?bandwidth)
                  (expected-bandwidth ?expected-bandwidth&~0.0))
  (object (is-a NODE) (node_id ?node0))
  (object (is-a NODE) (node_id ?node1))
  (test (< ?bandwidth ?expected-bandwidth))
  =>
  (bind ?severity (* (- 1 (gaussian ?bandwidth ?expected-bandwidth
                                    ?bandwidth)) 80))
  (make-instance of BOOLEAN_SIGN (node_id (create$ ?node0 ?node1))
                 (confidence 50) (severity ?severity)
                 (state observed) (source ?obj) (value TRUE)
                 (id "imb-pingpong-bandwidth-threshold")
                 (args (create$ ?bandwidth ?expected-bandwidth
                                (send ?obj get-impi-setting-string)))))

(defrule imb-pingpong-bandwidth-outlier
  "Check that the measured Intel(R) MPI Benchmark PingPong bandwidth
   is within the statistical range defined by other measured values in
   the same grouping."
  (object (name [config]) (clck-mode certification | health)
          (clck-checks $? imb_pingpong $?)
          (imb-pingpong-number-of-mad ?num-mad))
  ?obj <- (object (is-a IMB_PINGPONG) (node_id ?node0 ?node1) (bandwidth ?perf)
                  (arch ?arch ?arch) (endpoints remote)
                  (statistics_control TRUE))
  (object (is-a NODE) (node_id ?node0))
  (object (is-a NODE) (node_id ?node1))
  (test (or
         (> ?perf (+ (send ?obj median (send ?obj statistics-key bandwidth))
                     (* ?num-mad (send ?obj mad
                                       (send ?obj statistics-key bandwidth)))))
         (< ?perf (- (send ?obj median (send ?obj statistics-key bandwidth))
                     (* ?num-mad (send ?obj mad
                                       (send ?obj statistics-key bandwidth)))))))
  =>
  (bind ?key (send ?obj statistics-key bandwidth))
  (bind ?severity (* (- 1 (gaussian ?perf
                                    (send ?obj median ?key)
                                    (* 3 1.4826 (send ?obj mad ?key))))
                     80))
  (make-instance of BOOLEAN_SIGN (node_id (create$ ?node0 ?node1))
                 (confidence 50) (severity ?severity)
                 (state observed) (source ?obj) (value TRUE)
                 (id "imb-pingpong-bandwidth-outlier")
                 (args (create$ ?perf
                                (send ?obj median ?key)
                                (* ?num-mad (send ?obj mad ?key))
                                (send ?obj get-impi-setting-string)))))

(defrule imb-pingpong-latency-threshold
  "Check that the measured Intel(R) MPI Benchmark PingPong latency is
   less than or equal to the expected latency."
  (object (name [config]) (clck-mode certification | health)
          (clck-checks $? imb_pingpong $?))
  ?obj <- (object (is-a IMB_PINGPONG) (node_id ?node0 ?node1)
                  (latency ?latency)
                  (expected-latency ?expected-latency&~1e6))
  (object (is-a NODE) (node_id ?node0))
  (object (is-a NODE) (node_id ?node1))
  (test (> ?latency ?expected-latency))
  =>
  (bind ?severity (* (- 1 (gaussian ?latency ?expected-latency
                                    ?latency)) 80))
  (make-instance of BOOLEAN_SIGN (node_id (create$ ?node0 ?node1))
                 (confidence 50) (severity ?severity)
                 (state observed) (source ?obj) (value TRUE)
                 (id "imb-pingpong-latency-threshold")
                 (args (create$ ?latency ?expected-latency
                                (send ?obj get-impi-setting-string)))))

(defrule imb-pingpong-latency-outlier
  "Check that the measured Intel(R) MPI Benchmark PingPong latency is
   within the statistical range defined by other measured values in
   the same grouping."
  (object (name [config]) (clck-mode certification | health)
          (clck-checks $? imb_pingpong $?)
          (imb-pingpong-number-of-mad  ?num-mad))
  ?obj <- (object (is-a IMB_PINGPONG) (node_id ?node0 ?node1) (latency ?perf)
                  (arch ?arch ?arch) (endpoints remote)
                  (statistics_control TRUE))
  (object (is-a NODE) (node_id ?node0))
  (object (is-a NODE) (node_id ?node1))
  (test (or
         (> ?perf (+ (send ?obj median (send ?obj statistics-key latency))
                     (* ?num-mad (send ?obj mad
                                       (send ?obj statistics-key latency)))))
         (< ?perf (- (send ?obj median (send ?obj statistics-key latency))
                     (* ?num-mad (send ?obj mad
                                       (send ?obj statistics-key latency)))))))
  =>
  (bind ?key (send ?obj statistics-key latency))
  (bind ?severity (* (- 1 (gaussian ?perf
                                    (send ?obj median ?key)
                                    (* 3 1.4826 (send ?obj mad ?key))))
                     80))
  (make-instance of BOOLEAN_SIGN (node_id (create$ ?node0 ?node1))
                 (confidence 50) (severity ?severity)
                 (state observed) (source ?obj) (value TRUE)
                 (id "imb-pingpong-latency-outlier")
                 (args (create$ ?perf
                                (send ?obj median ?key)
                                (* ?num-mad (send ?obj mad ?key))
                                (send ?obj get-impi-setting-string)))))
