; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defrule infiniband-ca-type-is-not-consistent
  "Identify inconsistent InfiniBand HCA types."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? infiniband $?))
  ?obj <- (object (is-a INFINIBAND_HCA) (ca-type ?type)
                  (multiset_control TRUE) (node_id ?node))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj multiset-key ca-type) ?type)
           0.9))
  =>
  (bind ?key (send ?obj multiset-key ca-type))
  (bind ?fraction (- 1 (send ?obj fraction ?key ?type)))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence (* 100 ?fraction)) (severity 80)
                 (state observed) (source ?obj) (value TRUE)
                 (id "infiniband-ca-type-is-not-consistent")
                 (args (create$
                        (send ?obj get-ca)
                        (* 100 (send ?obj fraction ?key ?type))
                        ?type))))

(defrule infiniband-data-missing
  "Identify instances of missing InfiniBand information."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? infiniband $?))
  (object (is-a NODE) (node_id ?node) (architecture x86_64))
  (object (is-a INFINIBAND_PCI_DEVICE) (node_id ?node))
  (not (object (is-a SIGN) (id "infiniband-data-missing") (node_id ?node)))
  (or (not (object (is-a INFINIBAND_CONFIG) (node_id ?node)))
      (not (object (is-a INFINIBAND_HCA) (node_id ?node))))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 90)
                 (state observed) (value TRUE)
                 (id "infiniband-data-missing")
                 (remedy "infiniband-data-missing-remedy")))

(defrule infiniband-device-is-not-consistent
  "Identify inconsistent InfiniBand PCI devices."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? infiniband $?))
  ?obj <- (object (is-a INFINIBAND_PCI_DEVICE) (device ?d)
                  (multiset_control TRUE) (node_id ?node))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj multiset-key device) ?d) 0.9))
  =>
  (bind ?key (send ?obj multiset-key device))
  (bind ?fraction (- 1 (send ?obj fraction ?key ?d)))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence (* 100 ?fraction)) (severity 50)
                 (state observed) (source ?obj) (value TRUE)
                 (id "infiniband-device-is-not-consistent")
                 (args (create$
                        ?d
                        (* 100 (send ?obj fraction ?key ?d))))))

(defrule infiniband-driver-is-not-consistent
  "Identify inconsistent InfiniBand PCI drivers."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? infiniband $?))
  ?obj <- (object (is-a INFINIBAND_PCI_DEVICE) (driver ?d)
                  (multiset_control TRUE) (node_id ?node))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj multiset-key driver) ?d) 0.9))
  =>
  (bind ?key (send ?obj multiset-key driver))
  (bind ?fraction (- 1 (send ?obj fraction ?key ?d)))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence (* 100 ?fraction)) (severity 50)
                 (state observed) (source ?obj) (value TRUE)
                 (id "infiniband-driver-is-not-consistent")
                 (args (create$
                        (send ?obj get-device)
                        (send ?obj get-slot)
                        (* 100 (send ?obj fraction ?key ?d))
                        ?d))))

(defrule infiniband-firmware-version-is-not-consistent
  "Identify inconsistent InfiniBand HCA firmware versions."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? infiniband $?))
  ?obj <- (object (is-a INFINIBAND_HCA) (firmware-version ?v)
                  (multiset_control TRUE) (node_id ?node))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj multiset-key firmware-version) ?v)
           0.9))
  =>
  (bind ?key (send ?obj multiset-key firmware-version))
  (bind ?fraction (- 1 (send ?obj fraction ?key ?v)))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence (* 100 ?fraction)) (severity 50)
                 (state observed) (source ?obj) (value TRUE)
                 (id "infiniband-firmware-version-is-not-consistent")
                 (args (create$
                        (send ?obj get-ca)
                        (* 100 (send ?obj fraction ?key ?v))
                        ?v))))

(defrule infiniband-hardware-version-is-not-consistent
  "Identify inconsistent InfiniBand HCA hardware versions."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? infiniband $?))
  ?obj <- (object (is-a INFINIBAND_HCA) (hardware-version ?v)
                  (multiset_control TRUE) (node_id ?node))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj multiset-key hardware-version) ?v)
           0.9))
  =>
  (bind ?key (send ?obj multiset-key hardware-version))
  (bind ?fraction (- 1 (send ?obj fraction ?key ?v)))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence (* 100 ?fraction)) (severity 20)
                 (state observed) (source ?obj) (value TRUE)
                 (id "infiniband-hardware-version-is-not-consistent")
                 (args (create$
                        (send ?obj get-ca)
                        (* 100 (send ?obj fraction ?key ?v))
                        ?v))))

(defrule infiniband-physlot-is-not-consistent
  "Identify inconsistent InfiniBand PCI card physical slots."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? infiniband $?))
  ?obj <- (object (is-a INFINIBAND_PCI_DEVICE) (multiset_control TRUE)
                  (physlot ?p) (node_id ?node))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj multiset-key physlot) ?p) 0.9))
  =>
  (bind ?key (send ?obj multiset-key physlot))
  (bind ?fraction (- 1 (send ?obj fraction ?key ?p)))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence (* 100 ?fraction)) (severity 50)
                 (state observed) (source ?obj) (value TRUE)
                 (id "infiniband-physlot-is-not-consistent")
                 (args (create$
                        (send ?obj get-device)
                        (send ?obj get-slot)
                        (* 100 (send ?obj fraction
                                     (send ?obj multiset-key physlot) ?p))
                        ?p))))

(defrule infiniband-physical-state-is-not-consistent
  "Identify inconsistent InfiniBand HCA physical states."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? infiniband $?))
  ?obj <- (object (is-a INFINIBAND_HCA) (physical-state ?s)
                  (multiset_control TRUE) (node_id ?node))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj multiset-key physical-state) ?s)
           0.9))
  =>
  (bind ?key (send ?obj multiset-key physical-state))
  (bind ?fraction (- 1 (send ?obj fraction ?key ?s)))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence (* 100 ?fraction)) (severity 80)
                 (state observed) (source ?obj) (value TRUE)
                 (id "infiniband-physical-state-is-not-consistent")
                 (args (create$
                        (send ?obj get-ca)
                        (send ?obj get-port)
                        (* 100 (send ?obj fraction ?key ?s))
                        ?s))))

(defrule infiniband-rate-is-not-consistent
  "Identify inconsistent InfiniBand HCA rate."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? infiniband $?))
  ?obj <- (object (is-a INFINIBAND_HCA) (rate ?r)
                  (multiset_control TRUE) (node_id ?node))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj multiset-key rate) ?r)
           0.9))
  =>
  (bind ?key (send ?obj multiset-key rate))
  (bind ?fraction (- 1 (send ?obj fraction ?key ?r)))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence (* 100 ?fraction)) (severity 80)
                 (state observed) (source ?obj) (value TRUE)
                 (id "infiniband-rate-is-not-consistent")
                 (args (create$
                        (send ?obj get-ca)
                        (send ?obj get-port)
                        (* 100 (send ?obj fraction ?key ?r))
                        ?r))))

(defrule infiniband-rev-is-not-consistent
  "Identify inconsistent InfiniBand PCI card revision."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? infiniband $?))
  ?obj <- (object (is-a INFINIBAND_PCI_DEVICE) (multiset_control TRUE)
                  (rev ?r) (node_id ?node))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj multiset-key rev) ?r) 0.9))
  =>
  (bind ?key (send ?obj multiset-key rev))
  (bind ?fraction (- 1 (send ?obj fraction ?key ?r)))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence (* 100 ?fraction)) (severity 50)
                 (state observed) (source ?obj) (value TRUE)
                 (id "infiniband-rev-is-not-consistent")
                 (args (create$
                        (send ?obj get-device)
                        (send ?obj get-slot)
                        (* 100 (send ?obj fraction ?key ?r))
                        ?r))))

(defrule infiniband-state-is-not-consistent
  "Identify inconsistent InfiniBand HCA states."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? infiniband $?))
  ?obj <- (object (is-a INFINIBAND_HCA) (state ?s)
                  (multiset_control TRUE) (node_id ?node))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj multiset-key state) ?s)
           0.9))
  =>
  (bind ?key (send ?obj multiset-key state))
  (bind ?fraction (- 1 (send ?obj fraction ?key ?s)))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence (* 100 ?fraction)) (severity 80)
                 (state observed) (source ?obj) (value TRUE)
                 (id "infiniband-state-is-not-consistent")
                 (args (create$
                        (send ?obj get-ca)
                        (send ?obj get-port)
                        (* 100 (send ?obj fraction ?key ?s))
                        ?s))))

(defrule infiniband-memlock-is-not-consistent
  "Identify inconsistent memlock limits."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? infiniband $?))
  ?obj <- (object (is-a INFINIBAND_CONFIG) (node_id ?node)
                  (memlock ?memlock&~0) (multiset_control TRUE))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj multiset-key memlock) ?memlock) 0.9))
  =>
  (bind ?key (send ?obj multiset-key memlock))
  (bind ?fraction (- 1 (send ?obj fraction ?key ?memlock)))
  (make-instance of BOOLEAN_SIGN (node_id ?node) (confidence (* 100 ?fraction))
                 (severity 30) (state observed) (source ?obj) (value TRUE)
                 (id "infiniband-memlock-is-not-consistent")
                 (args (* 100 (send ?obj fraction ?key ?memlock))
                       ?memlock)))

(defrule infiniband-memlock-too-small
  "Identify too low memlock limits."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? infiniband $?))
  ?obj <- (object (is-a INFINIBAND_CONFIG) (node_id ?node)
                  (memlock ?memlock))
  (object (is-a NODE) (node_id ?node))
  (test (and (< ?memlock 200000) (!= ?memlock -1)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node) (confidence 90)
                 (severity 75) (state observed) (source ?obj) (value TRUE)
                 (id "infiniband-memlock-too-small")
                 (args ?memlock)
                 (remedy "infiniband-memlock-too-small-remedy")))

(defrule infiniband-ofed-version-is-not-consistent
  "Identify inconsistent OFED versions."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? infiniband $?))
  ?obj <- (object (is-a INFINIBAND_CONFIG) (node_id ?node)
                   (multiset_control TRUE) (ofed-version ?version))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj multiset-key ofed-version) ?version)
           0.9))
  =>
  (bind ?key (send ?obj multiset-key ofed-version))
  (bind ?fraction (- 1 (send ?obj fraction ?key ?version)))
  (make-instance of BOOLEAN_SIGN (node_id ?node) (confidence (* 100 ?fraction))
                 (severity 60) (state observed) (source ?obj) (value TRUE)
                 (id "infiniband-ofed-version-is-not-consistent")
                 (args (* 100 (send ?obj fraction ?key ?version))
                       ?version)))

(defrule infiniband-port-physical-state-not-linkup
  "Identify InfiniBand HCA ports not in the LinkUp physical state."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? infiniband $?))
  ?obj <- (object (is-a INFINIBAND_HCA) (physical-state ?s&~LinkUp)
                  (node_id ?node))
  (object (is-a NODE) (node_id ?node))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 65) (severity 80)
                 (state observed) (source ?obj) (value TRUE)
                 (id "infiniband-port-physical-state-not-linkup")
                 (args (create$ (send ?obj get-ca) (send ?obj get-port) ?s))))

(defrule infiniband-port-state-not-active
  "Identify InfiniBand HCA ports not in the Active state."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? infiniband $?))
  ?obj <- (object (is-a INFINIBAND_HCA) (state ?s&~Active) (node_id ?node))
  (object (is-a NODE) (node_id ?node))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 65) (severity 80)
                 (state observed) (source ?obj) (value TRUE)
                 (id "infiniband-port-state-not-active")
                 (args (create$ (send ?obj get-ca) (send ?obj get-port) ?s))))
