; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defrule iozone-ran-no-bandwidth
  "This rule fires on nodes that have bandwidth as 0.0. This is the
   default value and if this is the value found, it means the
   connector didn't find a regular expression match for the correct
   BW."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? iozone $?))
  ?obj <- (object (is-a IOZONE) (node_id ?hostname) (latest-timestamp TRUE)
                  (stdout ?out) (bandwidth 0))
  (object (is-a NODE) (node_id ?hostname))
  (test (> (str-length ?out) 0))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?hostname)
                 (confidence 60) (severity 90)
                 (state observed) (value TRUE) (source ?obj)
                 (id "iozone-ran-no-bandwidth")))

(defrule iozone-ran-not-complete
  "This rules fires on nodes where bandwidth is greater than
   0.0, (which means the test finished and the connector found a
   value) but the string 'iozone test complete' is missing from the
   output. This could mean a slow hard disk, or perhaps a failing hard
   disk, so let's warn the user"
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? iozone $?))
  ?obj <- (object (is-a IOZONE) (node_id ?hostname) (latest-timestamp TRUE)
                  (stdout ?out) (bandwidth ?bandwidth))
  (object (is-a NODE) (node_id ?hostname))
  (test (> ?bandwidth 0.0)) ; iozone ran and we got the bandwidth we want
  (test (eq (str-index "iozone test complete" ?out) FALSE))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?hostname)
                 (confidence 60) (severity 50)
                 (state observed) (value TRUE) (source ?obj)
                 (id "iozone-ran-not-complete")))

(defrule latest-iozone-is-too-old
  "Identify nodes where the most IOZONE data should be considered too
   old. Do not fire if there is not a recent heartbeat, i.e, the node
   is not returning any data and there is not a IOZONE specific
   issue. Too old is defined to mean no data from the last 7
   days (604800 seconds)"
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? iozone $?) (data-age-threshold ?age-threshold))
  ?obj <-(object (is-a IOZONE) (latest-timestamp TRUE) (node_id ?node)
                 (timestamp ?t))
  (object (is-a NODE) (node_id ?node))
  (not (object (is-a SIGN) (node_id ?node) (id "heartbeat-is-too-old")))
  (test (< ?t (- (now) ?age-threshold)))
  =>
  (bind ?age (round (- (now) ?t)))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 20)
                 (state observed) (value TRUE) (source ?obj)
                 (id "latest-iozone-is-too-old")
                 (args ?age)))

(defrule iozone-data-missing
  "Check that IOzone data is available."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? iozone $?))
  (object (is-a NODE) (node_id ?node) (architecture x86_64)
          (role $?role&:(member$ compute ?role)|:(member$ enhanced ?role)))
  (not (object (is-a IOZONE) (node_id ?node)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 90)
                 (state observed) (value TRUE)
                 (id "iozone-data-missing")
                 (remedy "iozone-data-missing-remedy")))

(defrule iozone-outlier
  "Locate values that are outliers.  An outlier is a value that is
   outside the range defined by the median +/- 6 * median absolute
   deviation.  The statistics are computed using all samples on all
   nodes (i.e., use the IOZONE statistics key).

   Note: the statistics_control condition is required to ensure that
   all samples are included when computing the statistics."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? iozone $?))
  ?obj <- (object (is-a IOZONE) (node_id ?node) (bandwidth ?bw)
                  (statistics_control TRUE) (latest-timestamp TRUE)
                  (valid TRUE))
  (object (is-a NODE) (node_id ?node))
  (test (or
         (> ?bw (+ (send ?obj median (send ?obj statistics-key))
                   (* 6 (send ?obj mad (send ?obj statistics-key)))))
         (< ?bw (- (send ?obj median (send ?obj statistics-key))
                   (* 6 (send ?obj mad (send ?obj statistics-key)))))))
  =>
  (bind ?key (send ?obj statistics-key))
  (bind ?median (send ?obj median ?key))
  (bind ?mad6 (* 6 (send ?obj mad ?key)))
  (bind ?severity (* (- 1 (gaussian ?bw ?median
                                    (* 3 1.4826 (send ?obj mad ?key)))) 80))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 75) (severity ?severity) (state observed)
                 (id "iozone-outlier") (source ?obj)
                 (args (create$ ?bw ?median ?mad6 ))))
