; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defrule kernel-data-missing
  "Check that kernel data is available."
  (object (is-a CONFIG) (name [config]) (clck-checks $? kernel $?))
  (object (is-a NODE) (node_id ?node))
  (not (object (is-a KERNEL) (node_id ?node)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 90)
                 (state observed) (value TRUE)
                 (id "kernel-data-missing")
                 (remedy "kernel-data-missing-remedy")))

(defrule kernel-not-icr
  "If the kernel version is less than 2.6.32 or not 2.6.18 with one of
   the accepted extensions: 238, 274, 308, 348, 371, then kernel is
   not Intel(R) Cluster Ready 1.3.1 compliant. If the base (everything
   before -) has letters, the connector will pass a flag ! to clips
   instead of the actual base version."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? kernel $?))
  ?obj <- (object (is-a KERNEL) (latest-timestamp TRUE) (node_id ?hostname)
                  (FullVersion ?fullV) (base ?ver) (extension ?ext))
  (object (is-a NODE) (node_id ?hostname) (architecture x86_64))
  (test
   (or (= (str-length ?ver) 0)
       (or
        (and
         (< (vercmp ?ver "2.6.32") 0)
         (<> (str-compare ?ver "2.6.18") 0))
        (and
         ( = (str-compare ?ver "2.6.18") 0 )
         (and
          (<> (str-compare ?ext "238") 0)
          (<> (str-compare ?ext "274") 0)
          (<> (str-compare ?ext "308") 0)
          (<> (str-compare ?ext "438") 0)
          (<> (str-compare ?ext "371") 0))))))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?hostname)
                 (confidence 90) (severity 80) (source ?obj)
                 (state observed) (value TRUE) (id "kernel-not-icr")
                 (args (create$ ?fullV))))

(defrule kernel-not-uniform
  "If the kernel version is not the same as at least 90% of the other nodes,
   then the node should be flagged as non-uniform.  The fewer other
   nodes that have the same kernel version increases the confidence
   that the node with the different version is incorrect."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? kernel $?))
  ?obj <- (object (is-a KERNEL) (latest-timestamp TRUE) (node_id ?node)
                  (FullVersion ?fullV&:(> (str-length ?fullV) 0))
                  (multiset_control TRUE))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj uniformity-key) ?fullV) 0.9))
  =>
  (bind ?fraction (* 100 (send ?obj fraction (send ?obj uniformity-key) ?fullV)))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence (- 100 ?fraction)) (severity 80) (source ?obj)
                 (state observed) (value TRUE) (id "kernel-not-uniform")
                 (args (create$ ?fraction ?fullV))))
