; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defrule lshw-data-missing
  "Check that lshw data is available."
  (object (is-a CONFIG) (name [config]) (clck-checks $? lshw $?))
  (object (is-a NODE) (node_id ?node) (architecture x86_64) (role $?role))
  (test (or (member$ compute $?role) (member$ enhanced $?role)))
  (not (object (is-a LSHW) (node_id ?node)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 90)
                 (state observed) (value TRUE)
                 (id "lshw-data-missing")
                 (remedy "lshw-data-missing-remedy")))

(defrule lshw-key-missing
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? lshw $?) (lshw-blacklist $?blacklist))
  ?obj <- (object (is-a LSHW) (node_id ?node) (key ?key)
                  (value ?value&:(= (str-length ?value) 0))
                  (builtin-blacklist $?builtin-blacklist))
  (object (is-a NODE) (node_id ?node) (architecture x86_64) (role $?role))
  (test (or (member$ compute $?role) (member$ enhanced $?role)))
  (test (= (regex_match ?key $?builtin-blacklist) 0))
  (test (= (regex_match ?key $?blacklist) 0))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node) (confidence 90) (severity 40)
                 (state observed) (source ?obj) (value TRUE)
                 (id "lshw-key-missing")
                 (args (create$ ?key))))

(defrule lshw-not-uniform
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? lshw $?)
          (lshw-blacklist $?blacklist) (lshw-uniformity-threshold ?thresh))
  ?obj <- (object (is-a LSHW) (node_id ?node) (key ?key)
                  (value ?value&:(!= (str-length ?value) 0))
                  (subcluster ?subcluster)
                  (multiset_control TRUE)
                  (builtin-blacklist $?builtin-blacklist))
  (object (is-a NODE) (node_id ?node) (architecture x86_64)
          (role $?role))
  (test (or (member$ compute $?role) (member$ enhanced $?role)))
  (test (= (regex_match ?key $?blacklist) 0))
  (test (= (regex_match ?key $?builtin-blacklist) 0))
  (test (< (send ?obj fraction (send ?obj uniformity-key) ?value) ?thresh))
  =>
  (bind ?fraction (* 100 (send ?obj fraction (send ?obj uniformity-key)
                               ?value)))

  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 40) (source ?obj)
                 (state observed) (value TRUE)
                 (id "lshw-not-uniform")
                 (args (create$ ?fraction ?key ?value ?subcluster))))
