; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defrule micinfo-data-missing
  "Check that micinfo data is available."
  (object (is-a CONFIG) (name [config]) (clck-checks $? micinfo $?))
  (object (is-a NODE) (node_id ?node) (architecture x86_64))
  (object (is-a XEON_PHI_PCI_DEVICE) (node_id ?node))
  (not (object (is-a MICINFO_HOST) (node_id ?node)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 90)
                 (state observed) (value TRUE)
                 (id "micinfo-data-missing")
                 (remedy "micinfo-data-missing-remedy")))

; Use this rule to set a failure flag
(defrule micinfo-host-driver-not-uniform
  "Checks the uniformity of the mpss driver that is installed on the
   host system."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? micinfo $?))
  ?obj <- (object (is-a MICINFO_HOST) (node_id ?node) (latest-timestamp TRUE)
                  (multiset_control TRUE)
                  (driver_version ?ver&:(!=(str-length ?ver) 0)))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj uniformity-key driver_version)
                 ?ver) 0.9))
  =>
  (bind ?confidence (* 100 (- 1 (send ?obj fraction
                                      (send ?obj uniformity-key driver_version)
                                      ?ver))))
  (make-instance of BOOLEAN_SIGN (node_id ?node) (confidence ?confidence)
                 (severity 50) (source ?obj) (state observed) (value TRUE)
                 (id "micinfo-host-driver-not-uniform")
                 (args (create$
                        (* 100 (send ?obj fraction
                                     (send ?obj uniformity-key driver_version)
                                     ?ver)) ?ver))))

(defrule micinfo-host-mpss-not-uniform
   "Checks the uniformity of mpss version that is installed on the host
    system."
   (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
           (clck-checks $? micinfo $?))
   ?obj <- (object (is-a MICINFO_HOST) (node_id ?node) (latest-timestamp TRUE)
                   (multiset_control TRUE)
                   (mpss_version ?ver&:(!=(str-length ?ver) 0)))
   (object (is-a NODE) (node_id ?node))
   (test (< (send ?obj fraction (send ?obj uniformity-key mpss_version)
                  ?ver) 0.9))
   =>
   (bind ?confidence (* 100 (- 1 (send ?obj fraction
                                       (send ?obj uniformity-key mpss_version)
                                       ?ver))))
   (make-instance of BOOLEAN_SIGN (node_id ?node) (confidence ?confidence)
                  (severity 50) (source ?obj) (state observed) (value TRUE)
                  (id "micinfo-host-mpss-not-uniform")
                  (args (create$
                         (* 100 (send ?obj fraction
                                      (send ?obj uniformity-key mpss_version)
                                      ?ver)) ?ver))))

(defrule micinfo-device-flash-not-uniform
  "Checks the uniformity of the flash versions that are installed on
   all of the k1om cards."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? micinfo $?))
  ?host <- (object (is-a MICINFO_HOST) (node_id ?node) (latest-timestamp TRUE)
                   (multiset_control TRUE))
  ?obj <- (object (is-a MICINFO_DEVICE) (host ?host) (device_id ?dev)
                  (multiset_control TRUE)
                  (Flash_Version ?ver&:(!=(str-length ?ver) 0)))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj uniformity-key Flash_Version)
                 ?ver) 0.9))
  =>
  (bind ?confidence (* 100 (- 1 (send ?obj fraction
                                      (send ?obj uniformity-key Flash_Version)
                                      ?ver))))
  (make-instance of BOOLEAN_SIGN (node_id ?node) (confidence ?confidence)
                 (severity 50) (source ?obj) (state observed) (value TRUE)
                 (id "micinfo-device-flash-not-uniform")
                 (args (create$
                        (* 100 (send ?obj fraction
                                     (send ?obj uniformity-key Flash_Version)
                                     ?ver)) ?ver ?dev))))

(defrule micinfo-device-smc-firmware-not-uniform
  "Checks the uniformity of smc firmware that is installed on all of the
   k1om cards."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? micinfo $?))
  ?host <- (object (is-a MICINFO_HOST) (node_id ?node) (latest-timestamp TRUE)
                   (multiset_control TRUE))
  ?obj <- (object (is-a MICINFO_DEVICE) (host ?host) (device_id ?dev)
                  (multiset_control TRUE)
                  (SMC_Firmware_Version ?ver&:(!=(str-length ?ver) 0)))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj uniformity-key SMC_Firmware_Version)
                 ?ver) 0.9))
  =>
  (bind ?confidence (* 100
                       (- 1 (send ?obj fraction
                                  (send ?obj uniformity-key SMC_Firmware_Version)
                                  ?ver))))
  (make-instance of BOOLEAN_SIGN (node_id ?node) (confidence ?confidence)
                 (severity 50) (source ?obj) (state observed) (value TRUE)
                 (id "micinfo-device-smc-firmware-not-uniform")
                 (args (create$
                        (* 100
                           (send ?obj fraction
                                 (send ?obj uniformity-key SMC_Firmware_Version)
                                 ?ver)) ?ver ?dev))))

(defrule micinfo-device-smc-boot-loader-not-uniform
  "Checks the uniformity of the smc boot loader versions installed on
   all of the k1om cards."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? micinfo $?))
  ?host <- (object (is-a MICINFO_HOST) (node_id ?node) (latest-timestamp TRUE)
                   (multiset_control TRUE))
  ?obj <- (object (is-a MICINFO_DEVICE) (host ?host) (device_id ?dev)
                  (multiset_control TRUE)
                  (SMC_Boot_Loader_Version ?ver&:(!=(str-length ?ver) 0)))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction
                 (send ?obj uniformity-key SMC_Boot_Loader_Version) ?ver) 0.9))
  =>
  (bind ?confidence
        (* 100
           (- 1 (send ?obj fraction
                      (send ?obj uniformity-key SMC_Boot_Loader_Version)
                      ?ver))))
  (make-instance of BOOLEAN_SIGN (node_id ?node) (confidence ?confidence) (source ?obj)
                 (severity 50)  (state observed) (value TRUE)
                 (id "micinfo-device-smc-boot-loader-not-uniform")
                 (args (create$
                        (* 100
                           (send ?obj fraction
                                 (send ?obj uniformity-key
                                       SMC_Boot_Loader_Version) ?ver))
                        ?ver ?dev))))

(defrule micinfo-device-uos-not-uniform
  "Checks the uniformity of the uOS version installed on all of the
   k1om cards."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? micinfo $?))
  ?host <- (object (is-a MICINFO_HOST) (node_id ?node) (latest-timestamp TRUE)
                   (multiset_control TRUE))
  ?obj <- (object (is-a MICINFO_DEVICE) (host ?host) (device_id ?dev)
                  (multiset_control TRUE)
                  (uOS_Version ?ver&:(!=(str-length ?ver) 0)))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj uniformity-key uOS_Version) ?ver)
           0.9))
  =>
  (bind ?confidence (* 100 (- 1 (send ?obj fraction
                                      (send ?obj uniformity-key uOS_Version)
                                      ?ver))))
  (make-instance of BOOLEAN_SIGN (node_id ?node) (confidence ?confidence)
                 (severity 50) (source ?obj) (state observed) (value TRUE)
                 (id "micinfo-device-uos-not-uniform")
                 (args (create$
                        (* 100 (send ?obj fraction
                                     (send ?obj uniformity-key uOS_Version)
                                     ?ver)) ?ver ?dev))))

(defrule micinfo-device-stepping-not-uniform
  "Checks the uniformity of the stepping of all of the k1om cards
   installed in the cluster."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? micinfo $?))
  ?host <- (object (is-a MICINFO_HOST) (node_id ?node) (latest-timestamp TRUE)
                   (multiset_control TRUE))
  ?obj <- (object (is-a MICINFO_DEVICE) (host ?host) (device_id ?dev)
                  (multiset_control TRUE)
                  (Coprocessor_Stepping ?ver&:(!=(str-length ?ver) 0)))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction
                 (send ?obj uniformity-key Coprocessor_Stepping) ?ver) 0.9))
  =>
  (bind ?confidence
        (* 100 (- 1 (send ?obj fraction
                          (send ?obj uniformity-key Coprocessor_Stepping)
                          ?ver))))
  (make-instance of BOOLEAN_SIGN (node_id ?node) (confidence ?confidence)
                 (severity 25) (state observed) (value TRUE) (source ?obj)
                 (id "micinfo-device-stepping-not-uniform")
                 (args (create$
                        (* 100 (send ?obj fraction
                                     (send ?obj uniformity-key Coprocessor_Stepping) ?ver))
                        ?ver ?dev))))

(defrule micinfo-die-temp-to-high
  "Checks that the die tempurature is not out of the safe operation bounds."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? micinfo $?))
  ?host <- (object (is-a MICINFO_HOST)(node_id ?node)(latest-timestamp TRUE))
  ?obj <- (object (is-a MICINFO_DEVICE) (host ?host) (device_id ?dev)
                  (Die_Temp ?temp))
  (object (is-a NODE) (node_id ?node))
  (test (> ?temp 90))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node) (confidence 90) (source ?obj)
                 (severity 99) (state observed) (value TRUE)
                 (id "micinfo-device-die-temp-to-high")
                 (args (create$ ?temp ?dev))))
