; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defrule mount-data-missing
  "Check that mount data is available."
  (object (is-a CONFIG) (name [config]) (clck-checks $? mount $?))
  (object (is-a NODE) (node_id ?node))
  (not (object (is-a MOUNT) (node_id ?node)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 90)
                 (state observed) (value TRUE)
                 (id "mount-data-missing")
                 (remedy "mount-data-missing-remedy")))

(defrule mount-proc-not-mounted
  "Check that /proc is properly mounted"
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? mount $?))
  ?obj <- (object (is-a MOUNT) (node_id ?node) (proc_mounted ?pm))
  (object (is-a NODE) (node_id ?node) (architecture x86_64))
  (test (eq ?pm 0))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 90)
                 (state observed) (source ?obj) (value TRUE)
                 (id "mount-proc-not-mounted")
                 (remedy "mount-proc-not-mounted-remedy")))

(defrule mount-dev-shm-not-mounted
  "Check that /dev/shm is properly mounted"
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? mount $?))
  ?obj <- (object (is-a MOUNT) (node_id ?node)
                  (dev_shm_mounted ?sm))
  (object (is-a NODE) (node_id ?node) (architecture x86_64))
  (test (eq ?sm 0))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 50)
                 (state observed) (source ?obj) (value TRUE)
                 (id "mount-dev-shm-not-mounted")
                 (remedy "mount-dev-shm-not-mounted-remedy")))

(defrule mount-bad-tmp-perms
  "Check that /tmp has the permissions 777."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? mount $?))
  ?obj <- (object (is-a MOUNT) (node_id ?node)
                  (tmp_perms ?perms))
  (object (is-a NODE) (node_id ?node) (architecture x86_64))
  (test (not (or (eq ?perms "0777") (eq ?perms "1777") (eq ?perms "777"))))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 10)
                 (state observed) (source ?obj) (value TRUE)
                 (id "mount-bad-tmp-perms")
                 (args (create$ ?perms))
                 (remedy "mount-bad-tmp-perms-remedy")))

(defrule mount-not-uniform-home-inode
  "Check that the home path is shared on the cluster by checking the
   uniformity of the inodes of the home directory."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? mount $?))
  ?obj <- (object (is-a MOUNT) (node_id ?node)
                  (multiset_control TRUE)
                  (home_inode ?home_inode&:(!= (str-length ?home_inode) 0)))
  (test (< (send ?obj fraction (sym-cat (send ?obj uniformity-key) -home_inode)
                 ?home_inode) 0.9))
  (object (is-a NODE) (node_id ?node) (architecture x86_64))
  =>
  (bind ?fraction (* 100 (send ?obj fraction
                               (sym-cat (send ?obj uniformity-key) -home_inode)
                               ?home_inode)))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence (- 100 ?fraction)) (severity 80) (source ?obj)
                 (state observed) (value TRUE)
                 (id "mount-not-uniform-home-inode")
                 (args (create$ ?fraction ?home_inode))))

(defrule mount-not-uniform-home-path
  "Check that the home path is uniform on the cluster."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? mount $?))
  ?obj <- (object (is-a MOUNT) (node_id ?node)
                  (multiset_control TRUE)
                  (home_path ?path&:(!=(str-length ?path) 0)))
  (test (< (send ?obj fraction (sym-cat (send ?obj uniformity-key) -home_path)
                 ?path) 0.9))
  (object (is-a NODE) (node_id ?node) (architecture x86_64))
  =>
  (bind ?fraction (* 100 (send ?obj fraction
                               (sym-cat (send ?obj uniformity-key) -home_path)
                               ?path)))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence (- 100 ?fraction)) (severity 80) (source ?obj)
                 (state observed) (value TRUE)
                 (id "mount-not-uniform-home-path")
                 (args (create$ ?fraction ?path))))
