; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defrule mpi_local-data-missing
  "Check that MPI intra-node data is available."
  (object (is-a CONFIG) (name [config]) (clck-checks $? mpi_local $?))
  (object (is-a NODE) (node_id ?node)
          (role $?role&:(member$ compute ?role)|:(member$ enhanced ?role)))
  (not (object (is-a MPI_LOCAL) (node_id ?node)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 90)
                 (state observed) (value TRUE)
                 (id "mpi_local-data-missing")
                 (remedy "mpi_local-data-missing-remedy")))

(defrule mpi-local-broken
  "Identify cases where there are less than 4 lines of valid output in
   the parsed output, but a mpirun binary executable was found"
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? mpi_local $?))
  ?obj <- (object (is-a MPI_LOCAL) (latest-timestamp TRUE) (node_id ?node)
                  (valid_lines ?valid) (mpi_path ?p))
  (object (is-a NODE) (node_id ?node))
  (test (!= ?valid 4))
  (test (> (str-length ?p) 0))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 90)
                 (source ?obj) (state observed) (value TRUE)
                 (id "mpi-local-broken")))

(defrule mpi-local-not-found
  "Identify cases where a mpirun binary executable itself was not found."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? mpi_local $?))
  ?obj <- (object (is-a MPI_LOCAL) (latest-timestamp TRUE) (node_id ?node)
                  (mpi_path ?p))
  (object (is-a NODE) (node_id ?node))
  (test (= (str-length ?p) 0))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 80) (severity 80)
                 (source ?obj) (state observed) (value TRUE)
                 (id "mpi-local-mpirun-not-found")
                 (remedy "mpi-local-mpirun-not-found-remedy")))

(defrule mpi-local-path-not-uniform
  "If the mpi-local-path found on each node is not the same as atleast
   90% of the other nodes, then the node should be flagged as
   non-uniform. The fewer other nodes that have the same
   mpi-local-path increase the confidence that the node with the
   different version is incorrect"
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? mpi_local $?))
  ?obj <- (object (is-a MPI_LOCAL) (latest-timestamp TRUE) (node_id ?node)
                  (mpi_path ?p&:(> (str-length ?p) 0)) (multiset_control TRUE))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj uniformity-key) ?p) 0.9))
  =>
  (bind ?percent (* 100 (send ?obj fraction (send ?obj uniformity-key) ?p)))
  (make-instance of BOOLEAN_SIGN (node_id ?node) (severity 60) (source ?obj)
                 (confidence (- 100 ?percent)) (state observed) (value TRUE)
                 (id "mpi-local-path-not-uniform")
                 (args (create$ ?percent ?p))))
