; Copyright 2006-2014 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defrule ntp-data-missing
  "Check that ntp data is available."
  (object (is-a CONFIG) (name [config]) (clck-checks $? ntp $?))
  (object (is-a NODE) (node_id ?node) (architecture x86_64))
  (not (object (is-a NTP) (node_id ?node)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 90)
                 (state observed) (value TRUE)
                 (id "ntp-data-missing")
                 (remedy "ntp-data-missing-remedy")))

(defrule ntp-offset-above-threshold
  "Check if reported time offset is larger than a threshold.
   Increase severity based on the size of the difference between the
   offset and threshold."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? ntp $?)
          (ntp-offset-threshold ?thresh))
  ?obj <- (object (is-a NTP) (node_id ?node) (remote ?remote) (offset ?offset))
  (object (is-a NODE) (node_id ?node))
  (test (> (abs ?offset) ?thresh))
  =>
  (bind ?severity (* (- 1 (gaussian (abs ?offset) ?thresh
                                    (/ (abs ?offset) 2))) 50))
  (make-instance of BOOLEAN_SIGN
                 (node_id ?node) (confidence 20) (severity ?severity)
                 (state observed) (value TRUE) (source ?obj)
                 (args (create$ ?remote ?offset ?thresh))
                 (id "ntp-offset-above-threshold")))

(defrule ntp-not-connected
  "Check if ntp client is not connected to an ntp server.  This is true
   if the remote slot is set to the default"
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? ntp $?))
  ?obj <- (object (is-a NTP) (node_id ?node)
                  (remote ?remote&:(= (str-length ?remote) 0)))
  (object (is-a NODE) (node_id ?node))
  =>
  (make-instance of BOOLEAN_SIGN
                 (node_id ?node) (confidence 30) (severity 50)
                 (state observed) (value TRUE) (source ?obj)
                 (id "ntp-not-connected")))
