; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defrule process-data-missing
  "Check that process data is available."
  (object (is-a CONFIG) (name [config]) (clck-checks $? process $?))
  (object (is-a NODE) (node_id ?node))
  (not (object (is-a PROCESS) (node_id ?node)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 90)
                 (state observed) (value TRUE)
                 (id "process-data-missing")
                 (remedy "process-data-missing-remedy")))

(defrule process-is-a-zombie
  "For the most recent PROCESS data point, identify nodes with zombie
   processes, i.e., processes with a 'Z' state."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? process $?))
  ?obj <- (object (is-a PROCESS) (node_id ?node) (etime ?etime)
                  (latest-timestamp TRUE) (pid ?pid) (state ?state) (uid ?uid)
                  (username ?uname) (cmd ?command))
  (object (is-a NODE) (node_id ?node))
  (test (= (str-compare ?state "Z") 0))
  =>
  ; the longer the zombie has been running, the higher the severity
  (bind ?severity (* (min 1 (/ ?etime 3600)) 100))
  (make-instance of BOOLEAN_SIGN
                 (node_id ?node) (confidence 90) (severity ?severity)
                 (state observed) (value TRUE) (source ?obj)
                 (id "process-is-a-zombie")
                 (args (create$ ?command ?uname ?etime ?pid))
                 (remedy "process-is-zombie-remedy")))

(defrule process-is-high-cpu
  "For the most recent PROCESS data point, identify nodes with high
   cpu processes, i.e., processes using more than 20% of a cpu core."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? process $?))
  ?obj <- (object (is-a PROCESS) (node_id ?node)
                  (latest-timestamp TRUE) (pcpu ?pcpu) (pid ?pid)
                  (timestamp ?t) (uid ?uid) (username ?uname)
                  (cmd ?command))
  (object (is-a NODE) (node_id ?node))
  (test (> ?pcpu 20))
  =>
  ; the higher the cpu percentage, the higher the severity
  (bind ?severity (min 100 ?pcpu))
  (make-instance of BOOLEAN_SIGN
                 (node_id ?node) (confidence 90) (severity ?severity)
                 (state observed) (value TRUE) (source ?obj)
                 (id "process-is-high-cpu")
                 (args (create$ ?command ?uname ?pcpu ?pid))
                 (remedy "process-is-high-cpu-remedy")))

(defrule process-is-high-memory
  "For the most recent PROCESS data point, identify nodes with high
   memory processes, i.e., processes using more than 50% of memory."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? process $?))
  ?obj <- (object (is-a PROCESS) (node_id ?node)
                  (latest-timestamp TRUE) (pid ?pid) (pmem ?pmem) (uid ?uid)
                  (username ?uname) (cmd ?command))
  (object (is-a NODE) (node_id ?node))
  (test (> ?pmem 50))
  =>
  ; the higher the memory percentage, the higher the severity
  (bind ?severity (min 100 ?pmem))
  (make-instance of BOOLEAN_SIGN
                 (node_id ?node) (confidence 90) (severity ?severity)
                 (state observed) (value TRUE) (source ?obj)
                 (id "process-is-high-memory")
                 (args (create$ ?command ?uname ?pmem ?pid))
                 (remedy "process-is-high-memory-remedy")))
