; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defrule rpm-data-missing
  "Check that RPM data is available."
  (object (is-a CONFIG) (name [config]) (clck-checks $? rpm $?))
  (object (is-a NODE) (node_id ?node) (role $?role))
  (not (object (is-a RPM) (node_id ?node)))
  (test (or (member$ compute $?role) (member$ enhanced $?role)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 90)
                 (state observed) (value TRUE)
                 (id "rpm-data-missing")
                 (remedy "rpm-data-missing-remedy")))

(defrule rpm-is-missing
  "Rule to determine if an rpm is missing on a node"
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? rpm $?) (rpm-uniformity-threshold ?thresh))
  ?obj <- (object (is-a RPM) (node_id ?node) (rpm ?rpm) (value ?v&FALSE)
                  (multiset_control TRUE))
  (object (is-a NODE) (node_id ?node) (architecture ?nodearch)
          (subcluster ?subcluster))
  (test (< (send ?obj fraction (sym-cat RPM - ?rpm - ?nodearch - ?subcluster
                                        -missing) ?v) ?thresh))
  =>
  (bind ?percent (* 100 (send ?obj fraction
                              (sym-cat RPM - ?rpm - ?nodearch - ?subcluster
                                       -missing) ?v)))
  (make-instance of BOOLEAN_SIGN (node_id ?node) (confidence (- 100 ?percent))
                 (severity 60) (source ?obj) (state observed) (value TRUE)
                 (id "rpm-is-missing")
                 (args (create$ ?rpm))))

(defrule rpm-is-extra
  "Rule to determine if a node has an extra rpm"
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? rpm $?) (rpm-uniformity-threshold ?thresh))

  ?obj <- (object (is-a RPM) (node_id ?node) (rpm ?rpm) (value ?v&TRUE)
                  (multiset_control TRUE))

  (object (is-a NODE) (node_id ?node) (architecture ?nodearch) (role $?role)
          (subcluster ?subcluster))

  (test (not (member$ head ?role)))
  (test (< (send ?obj fraction (sym-cat RPM - ?rpm - ?nodearch - ?subcluster
                                        -extra) ?v) ?thresh))
  =>
  (bind ?percent (* 100 (send ?obj fraction
                              (sym-cat RPM - ?rpm - ?nodearch - ?subcluster
                                       -extra) ?v)))
  (make-instance of BOOLEAN_SIGN (node_id ?node) (confidence (- 100 ?percent))
                 (severity 30) (source ?obj) (state observed) (value TRUE)
                 (id "rpm-is-extra")
                 (args (create$ ?rpm))))
