; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defrule shells-data-missing
  "Check that libraries data is available."
  (declare (salience ?*high-salience*))
  (object (is-a CONFIG) (name [config]) (clck-checks $? shells $?))
  (object (is-a NODE) (node_id ?node) (architecture x86_64))
  (not (object (is-a TOOL) (node_id ?node)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 90)
                 (state observed) (value TRUE)
                 (id "shells-data-missing")
                 (remedy "shells-data-missing-remedy")))

(defrule missing-sh
  (object (is-a CONFIG) (name [config]) (clck-mode health)
          (clck-checks $? shells $?))
  (object (is-a NODE) (node_id ?n) (architecture x86_64))
  (not (object (is-a SIGN) (id "shells-data-missing") (node_id ?n)))
  (not (object (is-a TOOL) (toolname sh) (node_id ?n)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?n)
                 (confidence 66) (severity 95) (value TRUE) (state observed)
                 (id "missing-shell") (args "sh")
                 (remedy "missing-shell-remedy")
                 (remedy-args "sh")))

(defrule missing-ksh
  (object (is-a CONFIG) (name [config]) (clck-mode health)
          (clck-checks $? shells $?))
  (object (is-a NODE) (node_id ?n) (architecture x86_64))
  (not (object (is-a SIGN) (id "shells-data-missing") (node_id ?n)))
  (not (object (is-a TOOL) (toolname ksh) (node_id ?n)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?n)
                 (confidence 99) (severity 10) (value TRUE) (state observed)
                 (id "missing-shell") (args "ksh")
                 (remedy "missing-shell-remedy")
                 (remedy-args "ksh")))

(defrule missing-bash
  (object (is-a CONFIG) (name [config]) (clck-mode health)
          (clck-checks $? shells $?))
  (object (is-a NODE) (node_id ?n) (architecture x86_64))
  (not (object (is-a SIGN) (id "shells-data-missing") (node_id ?n)))
  (not (object (is-a TOOL) (toolname bash) (node_id ?n)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?n)
                 (confidence 66) (severity 95) (value TRUE) (state observed)
                 (id "missing-shell") (args "bash")
                 (remedy "missing-shell-remedy")
                 (remedy-args "bash")))

(defrule missing-csh
  (object (is-a CONFIG) (name [config]) (clck-mode health)
          (clck-checks $? shells $?))
  (object (is-a NODE) (node_id ?n) (architecture x86_64))
  (not (object (is-a SIGN) (id "shells-data-missing") (node_id ?n)))
  (not (object (is-a TOOL) (toolname csh) (node_id ?n)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?n)
                 (confidence 80) (severity 75) (value TRUE) (state observed)
                 (id "missing-shell") (args "csh")
                 (remedy "missing-shell-remedy")
                 (remedy-args "csh")))

(defrule missing-tcsh
  (object (is-a CONFIG) (name [config]) (clck-mode health)
          (clck-checks $? shells $?))
  (object (is-a NODE) (node_id ?n) (architecture x86_64))
  (not (object (is-a SIGN) (id "shells-data-missing") (node_id ?n)))
  (not (object (is-a TOOL) (toolname tcsh) (node_id ?n)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?n)
                 (confidence 80) (severity 75) (value TRUE) (state observed)
                 (id "missing-shell") (args "tcsh")
                 (remedy "missing-shell-remedy")
                 (remedy-args "sh")))

 (defrule failing-sh
   (object (is-a CONFIG) (name [config]) (clck-mode health)
           (clck-checks $? shells $?))
   (object (is-a NODE) (node_id ?n) (architecture x86_64))
   ?obj <- (object (is-a TOOL) (toolname sh) (node_id ?n)
                   (stdout ?x&:(neq ?x "HelloWorld")))
   =>
   (make-instance of BOOLEAN_SIGN (node_id ?n)
                  (confidence 66) (severity 95)
                  (value TRUE) (state observed) (source ?obj)
                  (id "failing-shell") (args "sh")))

(defrule failing-ksh
  (object (is-a CONFIG) (name [config]) (clck-mode health)
          (clck-checks $? shells $?))
  (object (is-a NODE) (node_id ?n) (architecture x86_64))
  ?obj <- (object (is-a TOOL) (toolname ksh) (node_id ?n)
                  (stdout ?x&:(neq ?x "HelloWorld")))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?n)
                 (confidence 99) (severity 10)
                 (value TRUE) (state observed) (source ?obj)
                 (id "failing-shell") (args "ksh")))

(defrule failing-bash
  (object (is-a CONFIG) (name [config]) (clck-mode health)
          (clck-checks $? shells $?))
  (object (is-a NODE) (node_id ?n) (architecture x86_64))
  ?obj <- (object (is-a TOOL) (toolname bash) (node_id ?n)
                  (stdout ?x&:(neq ?x "HelloWorld")))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?n)
                 (confidence 66) (severity 95)
                 (value TRUE) (state observed) (source ?obj)
                 (id "failing-shell") (args "bash")))

(defrule failing-csh
  (object (is-a CONFIG) (name [config]) (clck-mode health)
          (clck-checks $? shells $?))
  (object (is-a NODE) (node_id ?n) (architecture x86_64))
  ?obj <- (object (is-a TOOL) (toolname csh) (node_id ?n)
                  (stdout ?x&:(neq ?x "HelloWorld")))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?n)
                 (confidence 80) (severity 75)
                 (value TRUE) (state observed) (source ?obj)
                 (id "failing-shell") (args "csh")))

(defrule failing-tcsh
  (object (is-a CONFIG) (name [config]) (clck-mode health)
          (clck-checks $? shells $?))
  (object (is-a NODE) (node_id ?n) (architecture x86_64))
  ?obj <- (object (is-a TOOL) (toolname tcsh) (node_id ?n)
                  (stdout ?x&:(neq ?x "HelloWorld")))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?n)
                 (confidence 80) (severity 75)
                 (value TRUE) (state observed) (source ?obj)
                 (id "failing-shell") (args "tcsh")))
