; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defrule storage-data-missing
  "Check that storage data is available."
  (object (is-a CONFIG) (name [config]) (clck-checks $? storage $?))
  (object (is-a NODE) (node_id ?node) (architecture x86_64))
  (not (object (is-a STORAGE) (node_id ?node)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 90)
                 (state observed) (value TRUE)
                 (id "storage-data-missing")
                 (remedy "storage-data-missing-remedy")))

(defrule storage-icr-compute
  "Checks the Intel(R) Cluster Ready required minimum for compute node storage.
   The head node must be attached to 20GiB of storage local or
   remote."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? storage $?))
  ?obj <- (object (is-a STORAGE) (node_id ?node)
                  (blocks_local ?blocks_local)
                  (blocks_network ?blocks_network))
  (object (is-a NODE) (node_id ?node) (architecture x86_64) (role $?role))
  (test (member$ compute ?role))
  (test (not (or (member$ head ?role) (member$ storage ?role))))
  (test (< (+ ?blocks_local ?blocks_network) 21474836480))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node) (confidence 50) (severity 80)
                 (state observed) (source ?obj) (value TRUE)
                 (id "storage-icr-compute")
                 (args (create$ (+ ?blocks_local ?blocks_network)
                                21474836480))))

(defrule storage-icr-head
  "Checks the Intel(R) Cluster Ready required minimum for head node storage.
   The head node must be attached to 200GiB of direct access storage."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? storage $?))
  ?obj <- (object (is-a STORAGE) (node_id ?node)
                  (blocks_local ?blocks_local))
  (object (is-a NODE) (node_id ?node) (architecture x86_64) (role $?role))
  (test (or (member$ head ?role) (member$ storage ?role)))
  (test (< ?blocks_local 214748364800))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node) (confidence 50) (severity 80)
                 (state observed) (source ?obj) (value TRUE)
                 (id "storage-icr-head")
                 (args (create$ ?blocks_local 214748364800))))
