; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defrule stream-data-missing
  "Check that STREAM data is available."
  (object (is-a CONFIG) (name [config]) (clck-checks $? stream $?))
  (object (is-a NODE) (node_id ?node)
          (role $?role&:(member$ compute ?role)|:(member$ enhanced ?role)))
  (not (object (is-a STREAM) (node_id ?node)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 90)
                 (state observed) (value TRUE)
                 (id "stream-data-missing")
                 (remedy "stream-data-missing-remedy")))

(defrule stream-no-runtimes
  "Look for cases where STREAM failed because libiomp5 could not be found."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? stream $?))
  ?obj <- (object (is-a STREAM) (node_id ?hostname) (latest-timestamp TRUE)
                  (stderr ?err))
  (object (is-a NODE) (node_id ?hostname))
  (test (str-index "libiomp5.so: cannot open shared object file: No such file or directory" ?err))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?hostname)
                 (confidence 20) (severity 20)
                 (state observed) (value TRUE) (source ?obj)
                 (id "stream-no-runtimes")))

(defrule stream-failed-validation
  "Look for cases where the string \"Failed validation\" is found in the STDOUT.
   In these cases, the triad value will still be populated though so
   we can't rely on the existence of the triad value"
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? stream $?))
  ?obj <- (object (is-a STREAM) (node_id ?hostname) (latest-timestamp TRUE)
                  (stdout ?out))
  (object (is-a NODE) (node_id ?hostname))
  (test (str-index "Failed Validation on" ?out))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?hostname)
                 (confidence 50) (severity 90)
                 (state observed) (value TRUE) (source ?obj)
                 (id "stream-failed-validation")))

(defrule latest-stream-data-is-too-old
  "Similar to latest-dgemm-data-is-too-old, just looks for out dated
   data"
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? stream $?) (data-age-threshold ?age-threshold))
  ?obj <-(object (is-a STREAM) (latest-timestamp TRUE) (node_id ?node)
                 (timestamp ?t))
  (object (is-a NODE) (node_id ?node))
  (not (object (is-a SIGN) (node_id ?node) (id "heartbeat-is-too-old")))
  (test (< ?t (- (now) ?age-threshold)))
  =>
  (bind ?age (round (- (now) ?t)))
  (make-instance of BOOLEAN_SIGN (node_id ?node) (confidence 90) (severity 20)
                 (state observed) (value TRUE) (source ?obj)
                 (id "latest-stream-data-is-too-old")
                 (args ?age)))

(defrule stream-outlier
  "Locate values that are outliers.  An outlier is a value that is
   outside the range defined by the median +/- 6 * median absolute
   deviation.  The statistics are computed using all samples on all
   nodes (i.e., use the STREAM statistics key).

   Note: the statistics_control condition is required to ensure that
   all samples are included when computing the statistics."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? stream $?))
  ?obj <- (object (is-a STREAM) (node_id ?node) (triad ?bw)
                  (statistics_control TRUE) (latest-timestamp TRUE)
                  (valid TRUE))
  (object (is-a NODE) (node_id ?node))
  (test (or
         (> ?bw (+ (send ?obj median (send ?obj statistics-key))
                   (* 6 (send ?obj mad (send ?obj statistics-key)))))
         (< ?bw (- (send ?obj median (send ?obj statistics-key))
                   (* 6 (send ?obj mad (send ?obj statistics-key)))))))
  =>
  (bind ?key (send ?obj statistics-key))
  (bind ?median (send ?obj median ?key))
  (bind ?mad6 (* 6 (send ?obj mad ?key)))
  (bind ?severity (* (- 1 (gaussian ?bw ?median
                                    (* 3 1.4826 (send ?obj mad ?key)))) 80))
  (make-instance of BOOLEAN_SIGN (node_id ?node) (source ?obj)
                 (confidence 75) (severity ?severity) (state observed)
                 (id "stream-outlier")
                 (args (create$ ?bw ?median ?mad6 ))))
