; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defrule x11_tools-data-missing
  "Check that X11 tool data is available."
  (declare (salience ?*high-salience*))
  (object (is-a CONFIG) (name [config]) (clck-checks $? x11_tools $?))
  (object (is-a NODE) (node_id ?node) (architecture x86_64)
          (role $? head $?))
  (not (object (is-a TOOL) (node_id ?node)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence 90) (severity 90)
                 (state observed) (value TRUE)
                 (id "x11_tools-data-missing")
                 (remedy "x11_tools-data-missing-remedy")))

(defrule missing-glxinfo
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? x11_tools $?))
  ?obj <- (object (is-a NODE) (node_id ?n) (architecture x86_64)
                  (role $? head $?))
  (not (object (is-a SIGN) (id "x11_tools-data-missing") (node_id ?n)))
  (not (object (is-a TOOL) (toolname glxinfo) (node_id ?n)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?n)
                 (confidence 90) (severity 80)
                 (value TRUE) (state observed) (source ?obj)
                 (id "missing-x11-tool") (args "glxinfo")
                 (remedy "missing-x11-tool-remedy")
                 (remedy-args "glxinfo")))

(defrule missing-xauth
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? x11_tools $?))
  ?obj <- (object (is-a NODE) (node_id ?n) (architecture x86_64)
                  (role $? head $?))
  (not (object (is-a SIGN) (id "x11_tools-data-missing") (node_id ?n)))
  (not (object (is-a TOOL) (toolname xauth) (node_id ?n)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?n)
                 (confidence 90) (severity 80)
                 (value TRUE) (state observed) (source ?obj)
                 (id "missing-x11-tool") (args "xauth")
                 (remedy "missing-x11-tool-remedy")
                 (remedy-args "xauth")))

(defrule missing-xev
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? x11_tools $?))
  ?obj <- (object (is-a NODE) (node_id ?n) (architecture x86_64)
                  (role $? head $?))
  (not (object (is-a SIGN) (id "x11_tools-data-missing") (node_id ?n)))
  (not (object (is-a TOOL) (toolname xev) (node_id ?n)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?n)
                 (confidence 90) (severity 80)
                 (value TRUE) (state observed) (source ?obj)
                 (id "missing-x11-tool") (args "xev")
                 (remedy "missing-x11-tool-remedy")
                 (remedy-args "xev")))

(defrule missing-xkill
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? x11_tools $?))
  ?obj <- (object (is-a NODE) (node_id ?n) (architecture x86_64)
                  (role $? head $?))
  (not (object (is-a SIGN) (id "x11_tools-data-missing") (node_id ?n)))
  (not (object (is-a TOOL) (toolname xkill) (node_id ?n)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?n)
                 (confidence 90) (severity 80)
                 (value TRUE) (state observed) (source ?obj)
                 (id "missing-x11-tool") (args "xkill")
                 (remedy "missing-x11-tool-remedy")
                 (remedy-args "xkill")))

(defrule missing-xterm
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? x11_tools $?))
  ?obj <- (object (is-a NODE) (node_id ?n) (architecture x86_64)
                  (role $? head $?))
  (not (object (is-a SIGN) (id "x11_tools-data-missing") (node_id ?n)))
  (not (object (is-a TOOL) (toolname xterm) (node_id ?n)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?n)
                 (confidence 90) (severity 80)
                 (value TRUE) (state observed) (source ?obj)
                 (id "missing-x11-tool") (args "xterm")
                 (remedy "missing-x11-tool-remedy")
                 (remedy-args "xterm")))

(defrule missing-xvinfo
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? x11_tools $?))
  ?obj <- (object (is-a NODE) (node_id ?n) (architecture x86_64)
                  (role $? head $?))
  (not (object (is-a SIGN) (id "x11_tools-data-missing") (node_id ?n)))
  (not (object (is-a TOOL) (toolname xvinfo) (node_id ?n)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?n)
                 (confidence 90) (severity 80)
                 (value TRUE) (state observed) (source ?obj)
                 (id "missing-x11-tool") (args "xvinfo")
                 (remedy "missing-x11-tool-remedy")
                 (remedy-args "xvinfo")))

(defrule missing-xwininfo
  (object (is-a CONFIG) (name [config]) (clck-mode certification | compliance)
          (clck-checks $? x11_tools $?))
  ?obj <- (object (is-a NODE) (node_id ?n) (architecture x86_64)
                  (role $? head $?))
  (not (object (is-a SIGN) (id "x11_tools-data-missing") (node_id ?n)))
  (not (object (is-a TOOL) (toolname xwininfo) (node_id ?n)))
  =>
  (make-instance of BOOLEAN_SIGN (node_id ?n)
                 (confidence 90) (severity 80)
                 (value TRUE) (state observed) (source ?obj)
                 (id "missing-x11-tool") (args "xwininfo")
                 (remedy "missing-x11-tool-remedy")
                 (remedy-args "xwininfo")))
