; Copyright 2006-2015 Intel Corporation All Rights Reserved.
;
; The source code contained or described herein and all documents
; related to the source code ("Material") are owned by Intel
; Corporation or its suppliers or licensors. Title to the Material
; remains with Intel Corporation or its suppliers and licensors. The
; Material may contain trade secrets and proprietary and confidential
; information of Intel Corporation and its suppliers and licensors,
; and is protected by worldwide copyright and trade secret laws and
; treaty provisions. No part of the Material may be used, copied,
; reproduced, modified, published, uploaded, posted, transmitted,
; distributed, or disclosed in any way without Intel's prior express
; written permission.
;
; No license under any patent, copyright, trade secret or other
; intellectual property right is granted to or conferred upon you by
; disclosure or delivery of the Materials, either expressly, by
; implication, inducement, estoppel or otherwise. Any license under
; such intellectual property rights must be express and approved by
; Intel in writing.
;
; Unless otherwise agreed by Intel in writing, you may not remove or
; alter this notice or any other notice embedded in Materials by Intel
; or Intel's suppliers or licensors in any way.

(defrule xeon-phi-device-is-not-consistent
 "Identify inconsistent Intel(R) Xeon Phi(TM) corpocessor PCI devices."
 (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
        (clck-checks $? miccheck $?))
  ?obj <- (object (is-a XEON_PHI_PCI_DEVICE) (device ?d)
                  (multiset_control TRUE) (node_id ?node))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj multiset-key device) ?d) 0.9))
  =>
  (bind ?key (send ?obj multiset-key device))
  (bind ?fraction (- 1 (send ?obj fraction ?key ?d)))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence (* 100 ?fraction)) (severity 50)
                 (state observed) (source ?obj) (value TRUE)
                 (id "xeon-phi-device-is-not-consistent")
                 (args (create$
                        ?d
                        (* 100 (send ?obj fraction ?key ?d))))))

(defrule xeon-phi-driver-is-not-consistent
  "Identify inconsistent XIntel(R) Xeon Phi(TM) corpocessor PCI drivers."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? miccheck $?))
  ?obj <- (object (is-a XEON_PHI_PCI_DEVICE) (driver ?d)
                  (multiset_control TRUE) (node_id ?node))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj multiset-key driver) ?d) 0.9))
  =>
  (bind ?key (send ?obj multiset-key driver))
  (bind ?fraction (- 1 (send ?obj fraction ?key ?d)))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence (* 100 ?fraction)) (severity 50)
                 (state observed) (source ?obj) (value TRUE)
                 (id "xeon-phi-driver-is-not-consistent")
                 (args (create$
                        (send ?obj get-device)
                        (send ?obj get-slot)
                        (* 100 (send ?obj fraction ?key ?d))
                        ?d))))

(defrule xeon-phi-physlot-is-not-consistent
  "Identify inconsistent XIntel(R) Xeon Phi(TM) corpocessor PCI card physical slots."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? miccheck $?))
  ?obj <- (object (is-a XEON_PHI_PCI_DEVICE) (multiset_control TRUE)
                  (physlot ?p) (node_id ?node))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj multiset-key physlot) ?p) 0.9))
  =>
  (bind ?key (send ?obj multiset-key physlot))
  (bind ?fraction (- 1 (send ?obj fraction ?key ?p)))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence (* 100 ?fraction)) (severity 50)
                 (state observed) (source ?obj) (value TRUE)
                 (id "xeon-phi-physlot-is-not-consistent")
                 (args (create$
                        (send ?obj get-device)
                        (send ?obj get-slot)
                        (* 100 (send ?obj fraction
                                     (send ?obj multiset-key physlot) ?p))
                        ?p))))

(defrule xeon-phi-rev-is-not-consistent
  "Identify inconsistent Intel(R) Xeon Phi(TM) corpocessor PCI card revision."
  (object (is-a CONFIG) (name [config]) (clck-mode certification | health)
          (clck-checks $? miccheck $?))
  ?obj <- (object (is-a XEON_PHI_PCI_DEVICE) (multiset_control TRUE)
                  (rev ?r) (node_id ?node))
  (object (is-a NODE) (node_id ?node))
  (test (< (send ?obj fraction (send ?obj multiset-key rev) ?r) 0.9))
  =>
  (bind ?key (send ?obj multiset-key rev))
  (bind ?fraction (- 1 (send ?obj fraction ?key ?r)))
  (make-instance of BOOLEAN_SIGN (node_id ?node)
                 (confidence (* 100 ?fraction)) (severity 50)
                 (state observed) (source ?obj) (value TRUE)
                 (id "xeon-phi-rev-is-not-consistent")
                 (args (create$
                        (send ?obj get-device)
                        (send ?obj get-slot)
                        (* 100 (send ?obj fraction ?key ?r))
                        ?r))))


