#!/bin/bash

. $(dirname $0)/../../functions

mpivars

# This environment variable will be defined for us
# CLCK_DP_NODEFILE
if [ ! -f "${CLCK_DP_NODEFILE}" ] ; then
    echo "Missing or invalid nodefile: '${CLCK_DP_NODEFILE}'" >&2
    exit 1
fi

# Configure MPI to timeout before CLCK does to give MPI a chance to cleanup
if [[ ! -z $CLCK_TIMEOUT && $CLCK_TIMEOUT -gt 5 ]]; then
  export I_MPI_JOB_TIMEOUT=$(( $CLCK_TIMEOUT - 5 ))
fi

# Configure MPI to cleanup
export I_MPI_MPIRUN_CLEANUP=1

# Look for user configured options sent in via environment variables

if [ "$CLCK_PROVIDER_IMB_PINGPONG_FABRICS" == "" ]; then
    MY_FABRICS=""
else
    MY_FABRICS="-genv I_MPI_FABRICS=${CLCK_PROVIDER_IMB_PINGPONG_FABRICS}"
fi

if [ "$CLCK_PROVIDER_IMB_PINGPONG_DAPL_PROVIDER" == "" ]; then
    MY_DAPL=""
else
    MY_DAPL="-genv I_MPI_DAPL_PROVIDER=${CLCK_PROVIDER_IMB_PINGPONG_DAPL_PROVIDER}"
fi

if [ "$CLCK_PROVIDER_IMB_PINGPONG_OFA_ADAPTER_NAME" == "" ]; then
    MY_OFA=""
else
    MY_OFA="-genv I_MPI_OFA_ADAPTER_NAME=${CLCK_PROVIDER_IMB_PINGPONG_OFA_ADAPTER_NAME}"
fi

if [ "$CLCK_PROVIDER_IMB_PINGPONG_TCP_NETMASK" == "" ]; then
    MY_TCP=""
else
    MY_TCP="-genv I_MPI_TCP_NETMASK=${CLCK_PROVIDER_IMB_PINGPONG_TCP_NETMASK}"
fi

if [ "$CLCK_PROVIDER_IMB_PINGPONG_TMI_PROVIDER" == "" ]; then
    MY_TMI=""
else
    MY_TMI="-genv I_MPI_TMI_PROVIDER=${CLCK_PROVIDER_IMB_PINGPONG_TMI_PROVIDER}"
fi

if [ "$CLCK_PROVIDER_IMB_PINGPONG_OPTIONS" == "" ]; then
    MY_OPTIONS=""
else
    MY_OPTIONS=${CLCK_PROVIDER_IMB_PINGPONG_OPTIONS}
fi

# Note: If the user options has a I_MPI_DEBUG, we over ride it with 5 anyway
# Note: the ${CLCK_I_MPI_ROOT} should have been defined for us if the
#       mpivars function executed successfully.

# TODO: This should probably be split into multiple lines for easier parsing
#       and keeping it within 80 characters

I_MPI_MIC=enable mpirun $MY_OPTIONS $MY_DAPL $MY_OFA $MY_TCP $MY_TMI $MY_FABRICS -f ${CLCK_DP_NODEFILE} -ppn 1 -genv I_MPI_DEBUG 5 $(dirname $0)/imb_wrapper.sh ${CLCK_I_MPI_ROOT} pingpong -multi 1 -msglen $1
